import wx
import wx.lib.newevent

from minixs.gui.mouse_event import *

EventRangeActionChanged, EVT_RANGE_ACTION_CHANGED = wx.lib.newevent.NewCommandEvent()
EventRangeChanged, EVT_RANGE_CHANGED = wx.lib.newevent.NewCommandEvent()

class Tool(object):
  def __init__(self, parent):
    self.parent = parent
    self.parent.AddTool(self)

    self.active = False
    self.visible = True

  def SetActive(self, active):
    self.active = active

  def SetVisible(self, visible):
    if visible == self.visible:
      return

    self.visible = visible
    self.parent.Refresh()

  def OnLeftDown(self, evt):
    pass

  def OnLeftUp(self, evt):
    pass

  def OnRightDown(self, evt):
    pass

  def OnRightUp(self, evt):
    pass

  def OnMiddleDown(self, evt):
    pass

  def OnMiddleUp(self, evt):
    pass

  def OnMotion(self, evt):
    pass

  def OnEnterWindow(self, evt):
    pass

  def OnLeaveWindow(self, evt):
    pass

  def OnPaint(self, evt, dc):
    pass


class RangeTool(Tool):
  """
  A tool to allow selecting one or more ranges in one or more directions
  """
  VERTICAL = 1
  HORIZONTAL = 2

  ACTION_NONE = 0
  ACTION_RESIZE_L = 0x01
  ACTION_RESIZE_R = 0x02
  ACTION_RESIZE_T = 0x04
  ACTION_RESIZE_B = 0x08
  ACTION_MOVE     = 0x10
  ACTION_PROPOSED = 0x100

  ACTION_RESIZE_TL = ACTION_RESIZE_T | ACTION_RESIZE_L
  ACTION_RESIZE_TR = ACTION_RESIZE_T | ACTION_RESIZE_R
  ACTION_RESIZE_BL = ACTION_RESIZE_B | ACTION_RESIZE_L
  ACTION_RESIZE_BR = ACTION_RESIZE_B | ACTION_RESIZE_R

  ACTION_RESIZE = ACTION_RESIZE_L | ACTION_RESIZE_R | \
                  ACTION_RESIZE_T | ACTION_RESIZE_B

  def __init__(self, *args, **kwargs):
    """
    Initialize tool
    """
    Tool.__init__(self, *args, **kwargs)

    self.rects = []
    self.active_rect = None

    self.multiple = False
    self.direction = self.VERTICAL | self.HORIZONTAL

    self.action = self.ACTION_NONE

    self.brush = wx.Brush(wx.Colour(255,255,255,50))
    self.pen = wx.Pen('#ffff22', 1, wx.DOT_DASH)
    self.active_pen = wx.Pen('#33dd33', 1, wx.DOT_DASH)
    self.action_pen = wx.Pen('#22ffff', 2, wx.SOLID)

    self.range_changed = False
    self.post_range_change_immediate = False

  def RangeChanged(self):
    if self.post_range_change_immediate:
      self.PostEventRangeChanged()
    else:
      self.range_changed = True

  def PostEventRangeChanged(self):
    """
    Send event indicating that selected range has changed
    """
    evt = EventRangeChanged(self.parent.Id, range=self.rects)
    wx.PostEvent(self.parent, evt)

  def PostEventRangeActionChanged(self, in_window):
    """
    Send event indicating that current action has changed
    """
    evt = EventRangeActionChanged(self.parent.Id,
        action=self.action,
        range=self.active_rect,
        in_window=in_window)
    wx.PostEvent(self.parent, evt)

  def DetermineAction(self, x, y):
    """
    Determine action to perform based on the provided location

    Parameters
    ----------
      x: x coordinate
      y: y coordinate

    Returns
    -------
      (rect, action)

      rect: the rectangle to act on
      action: the action to perform (a bitmask of self.ACTION_*)
    """

    off = 4
    action = self.ACTION_NONE
    active_rect = None

    # run through rects backwards (newest are on top)
    for rect in self.rects[::-1]:
      (x1,y1),(x2,y2) = rect

      # check if within offset of a rect edge, if so, resize
      if y1 - off < y < y2 + off:
        if abs(x1 - x) < off:
          action |= self.ACTION_RESIZE_L
          active_rect = rect
        elif abs(x2 - x) < off:
          action |= self.ACTION_RESIZE_R
      if x1 - off < x < x2 + off:
        if abs(y1 - y) < off:
          action |= self.ACTION_RESIZE_T
        elif abs(y2 - y) < off:
          action |= self.ACTION_RESIZE_B

      # not close to edge, but within rect => move
      if action == self.ACTION_NONE:
        if x1 < x < x2 and y1 < y < y2:
          action = self.ACTION_MOVE

      # only perform actions commensurate with direction
      mask = self.ACTION_MOVE
      if self.direction & self.VERTICAL:
        mask |= self.ACTION_RESIZE_T
        mask |= self.ACTION_RESIZE_B
      if self.direction & self.HORIZONTAL:
        mask |= self.ACTION_RESIZE_L
        mask |= self.ACTION_RESIZE_R
      action &= mask

      if action != self.ACTION_NONE:
        return (rect, action)

    return (None, self.ACTION_NONE)

  def SetMultiple(self, multiple):
    self.multiple = multiple

  def SetDirection(self, direction):
    self.direction = direction
    self.parent.Refresh()

  def ToggleDirection(self, direction, on=None):
    """
    Toggle range direction

    Parameters
    ----------
      direction: Crosshair.VERTICAL or .HORIZONTAL
      on: True for on, False for off, or None for toggle 

    Note: the directions can be bitwise or'd together. (e.g. RangeTool.VERTICAL | RangeTool.HORIZONTAL)
    """

    if on is None:
      self.direction ^= direction
    elif on:
      self.direction |= direction
    else:
      self.direction &= ~direction

  def OnLeftDown(self, evt):
    """
    Handle left mouse down
    """

    if mouse_event_modifier_mask(evt) != MOD_NONE:
      return

    x, y = evt.GetPosition()
    x, y = self.parent.CoordScreenToBitmap(x,y)

    w, h = self.parent.GetBitmapSize()

    if self.action & self.ACTION_PROPOSED:
      self.action &= ~self.ACTION_PROPOSED
      self.action_start = (x,y)
    else:
      if self.direction & self.VERTICAL:
        y1 = y
        y2 = y + 1
      else:
        y1 = 0
        y2 = h
      if self.direction & self.HORIZONTAL:
        x1 = x
        x2 = x + 1
      else:
        x1 = 0
        x2 = w

      rect = [[x1,y1],[x2,y2]]
      if self.multiple:
        self.rects.append(rect)
      else:
        self.rects = [rect]

      self.active_rect = rect
      self.action = self.ACTION_NONE
      if self.direction & self.HORIZONTAL:
        self.action |= self.ACTION_RESIZE_R
      if self.direction & self.VERTICAL:
        self.action |= self.ACTION_RESIZE_B

    self.parent.Refresh()

  def OnLeftUp(self, evt):
    """
    Handle left mouse up
    """

    if mouse_event_modifier_mask(evt) != MOD_NONE:

      # not all computers (Macs) have middle mouse buttons, so let
      # shift-left-click
      if evt.ShiftDown():
        self.SplitActiveRect(evt.ControlDown())
      return

    x,y = evt.GetPosition()
    x,y = self.parent.CoordScreenToBitmap(x,y)
    (x1,y1), (x2,y2) = self.active_rect

    if self.action & self.ACTION_RESIZE:
      # normalize rect coords so x1<x2 and y1<y2
      if x2 < x1:
        self.active_rect[0][0], self.active_rect[1][0] = x2, x1
      if y2 < y1:
        self.active_rect[0][1], self.active_rect[1][1] = y2, y1

      # don't keep rects with vanishing size
      if abs(x2 - x1) < 2 or abs(y2 - y1) < 2:
        self.rects.remove(self.active_rect)
        self.parent.Refresh()

    rect, action = self.DetermineAction(x, y)
    if action != self.ACTION_NONE:
      action |= self.ACTION_PROPOSED
    self.active_rect = rect
    self.action = action 

    if self.range_changed:
      self.PostEventRangeChanged()
      self.range_changed = False

    self.parent.Refresh()

  def OnRightUp(self, evt):
    """
    Handle right mouse up
    """

    if mouse_event_modifier_mask(evt) != MOD_NONE:
      return

    if self.action & self.ACTION_PROPOSED:
      self.rects.remove(self.active_rect)
      self.active_rect = None
      self.action = self.ACTION_NONE
      self.PostEventRangeChanged()
      self.parent.Refresh()

  def OnMiddleUp(self, evt):
    """
    Handle middle mouse up
    """

    horizontal = evt.ControlDown()
    self.SplitActiveRect(horizontal)


  def SplitActiveRect(self, horizontal=False):
    """
    Split active rectangle in half.

    Parameters:
      horizontal: if True, split rect horizontally. else, split rect vertically
    """

    if self.action & self.ACTION_PROPOSED:
      r = self.active_rect
      (x1,y1), (x2,y2) = r
      r2 = [[x1,y1],[x2,y2]]

      # split active rect in half
      if horizontal:
        #split horizontally
        (x1,y1), (x2,y2) = r
        yp = (y1 + y2) / 2.0
        r[1][1] = yp-1
        r2[0][1] = yp+1
      else:
        #split vertically
        xp = (x1 + x2) / 2.0
        r[1][0] = xp-2
        r2[0][0] = xp+2

      self.rects.append(r2)
      self.PostEventRangeChanged()
      self.parent.Refresh()

  def OnMotion(self, evt):
    """
    Handle mouse motion
    """
    x, y = evt.GetPosition()
    x, y = self.parent.CoordScreenToBitmap(x,y)

    w, h = self.parent.GetBitmapSize()

    # not currently performing an action
    if self.action == self.ACTION_NONE or self.action & self.ACTION_PROPOSED:
      rect, action = self.DetermineAction(x,y)

      needs_refresh = False
      if self.action & ~self.ACTION_PROPOSED != action or rect != self.active_rect:
        needs_refresh = True

      if rect:
        action |= self.ACTION_PROPOSED

      if action != self.action or rect != self.active_rect or rect is None:
        self.action = action
        self.active_rect = rect

        self.PostEventRangeActionChanged(in_window=True)

      if needs_refresh:
        self.parent.Refresh()

    elif self.action & self.ACTION_RESIZE:
      # clamp mouse to within panel
      if x < 0: x = 0
      if x > w: x = w
      if y < 0: y = 0
      if y > h: y = h

      if self.action & self.ACTION_RESIZE_L:
        self.active_rect[0][0] = x
      elif self.action & self.ACTION_RESIZE_R:
        self.active_rect[1][0] = x
      if self.action & self.ACTION_RESIZE_T:
        self.active_rect[0][1] = y
      elif self.action & self.ACTION_RESIZE_B:
        self.active_rect[1][1] = y

      self.RangeChanged()
      self.parent.Refresh()

    elif self.action & self.ACTION_MOVE:
      (x1, y1), (x2, y2) = self.active_rect
      xs,ys = self.action_start
      dx, dy = x - xs, y - ys

      if dx < -x1: dx = -x1
      if dy < -y1: dy = -y1
      if dx > w-x2: dx = w-x2
      if dy > h-y2: dy = h-y2

      if self.direction & self.HORIZONTAL:
        self.active_rect[0][0] += dx
        self.active_rect[1][0] += dx
      if self.direction & self.VERTICAL:
        self.active_rect[0][1] += dy
        self.active_rect[1][1] += dy
      self.action_start = (x,y)

      self.RangeChanged()
      self.parent.Refresh()

  def OnEnterWindow(self, evt):
    """
    Handle entering window
    """
    pass

  def OnLeaveWindow(self, evt):
    """
    Handle leaving window
    """
    if self.action & self.ACTION_PROPOSED or self.action == self.ACTION_NONE:
      self.action = self.ACTION_NONE
      self.active_rect = None

      self.PostEventRangeActionChanged(in_window=False)
      self.parent.Refresh()

  def OnPaint(self, evt, dc):
    """
    Draw tool
    """
    gcdc = wx.GCDC(dc)

    dc.SetBrush(wx.TRANSPARENT_BRUSH)
    gcdc.SetBrush(self.brush)
    gcdc.SetPen(wx.TRANSPARENT_PEN)

    for r in self.rects:
      if r == self.active_rect:
        dc.SetPen(self.active_pen)
      else:
        dc.SetPen(self.pen)

      (x1,y1),(x2,y2) = r 

      # transform coords
      x1,y1 = self.parent.CoordBitmapToScreen(x1,y1)
      x2,y2 = self.parent.CoordBitmapToScreen(x2,y2)

      gcdc.DrawRectangle(x1,y1,x2-x1,y2-y1)
      dc.DrawRectangle(x1,y1,x2-x1+1,y2-y1+1)

      if self.active_rect and self.action & self.ACTION_RESIZE and self.action & self.ACTION_PROPOSED:

        dc.SetPen(self.action_pen)
        (x1,y1),(x2,y2) = self.active_rect
        x1,y1 = self.parent.CoordBitmapToScreen(x1,y1)
        x2,y2 = self.parent.CoordBitmapToScreen(x2,y2)

        if self.action & self.ACTION_RESIZE_L:
          dc.DrawLine(x1,y1,x1,y2)
        if self.action & self.ACTION_RESIZE_R:
          dc.DrawLine(x2,y1,x2,y2)
        if self.action & self.ACTION_RESIZE_T:
          dc.DrawLine(x1,y1,x2,y1)
        if self.action & self.ACTION_RESIZE_B:
          dc.DrawLine(x1,y2,x2,y2)


class Crosshair(Tool):
  VERTICAL = 1
  HORIZONTAL = 2

  def __init__(self, *args, **kwargs):
    Tool.__init__(self, *args, **kwargs)

    self.direction = self.VERTICAL | self.HORIZONTAL
    self.pos = None

    self.pen = wx.Pen('#222222', 1, wx.SOLID)

  def SetDirection(self, direction):
    self.direction = direction
    self.parent.Refresh()

  def ToggleDirection(self, direction, on=None):
    """
    Toggle crosshair direction

    Parameters
    ----------
      direction: Crosshair.VERTICAL or .HORIZONTAL
      on: True for on, False for off, or None for toggle 

    Note: the directions can be bitwise or'd together. (e.g. Crosshair.VERTICAL | Crosshair.HORIZONTAL)
    """

    if on is None:
      self.direction ^= direction
    elif on:
      self.direction |= direction
    else:
      self.direction &= ~direction

  def OnLeftDown(self, evt):
    pass

  def OnMotion(self, evt):
    self.pos = evt.GetPosition()
    self.parent.Refresh()

  def OnLeaveWindow(self, evt):
    self.pos = None
    self.parent.Refresh()

  def OnPaint(self, evt, dc):
    if self.pos is None:
      return

    dc.SetLogicalFunction(wx.INVERT)

    w, h = self.parent.GetSize()
    dc.SetPen(self.pen)

    if self.direction & self.VERTICAL:
      x1 = x2 = self.pos[0]
      y1 = 0
      y2 = h
      dc.DrawLine(x1,y1,x2,y2)

    if self.direction & self.HORIZONTAL:
      y1 = y2 = self.pos[1]
      x1 = 0
      x2 = w
      dc.DrawLine(x1,y1,x2,y2)

    dc.SetLogicalFunction(wx.COPY)



import wx
import minixs.filter as filter
from minixs.gui import filter_view

ID_MAIN_FRAME       = wx.NewId()
ID_SPECTROMETER     = wx.NewId()
ID_DATASET_NAME     = wx.NewId()
ID_EXPOSURE_LIST    = wx.NewId()
ID_READ_ENERGIES    = wx.NewId()
ID_SELECT_EXPOSURES = wx.NewId()
ID_APPEND_ROW       = wx.NewId()
ID_DELETE_ROW       = wx.NewId()
ID_CLEAR_ENERGIES   = wx.NewId()
ID_CLEAR_EXPOSURES  = wx.NewId()
ID_DISPERSIVE_DIR   = wx.NewId()
ID_EXPOSURE_SLIDER  = wx.NewId()
ID_CALIBRATE        = wx.NewId()
ID_FILTER_EMISSION  = wx.NewId()
ID_IMAGE_PANEL      = wx.NewId()

ID_VIEW_TYPE        = wx.NewId()
ID_SHOW_XTALS       = wx.NewId()
ID_FIND_XTALS       = wx.NewId()

ID_IMPORT_XTALS     = wx.NewId()
ID_EXPORT_XTALS     = wx.NewId()

ID_LOAD_SCAN        = wx.NewId()

STATUS_COORDS  = 0
STATUS_MESSAGE = 1




import wx

from wx.lib.scrolledpanel import ScrolledPanel
from minixs import filter
from minixs import spectrometer
from minixs.gui.frame import MenuFrame
from minixs.gui.image_view import ImageView
from minixs.gui import filter_view

from const import *

import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import \
    FigureCanvasWxAgg as FigureCanvas,  \
    NavigationToolbar2WxAgg

HPAD = 10
VPAD = 5


class Panel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

class ToolsPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    self.pages = []
    page = ToolsPanelCombined(self, wx.ID_ANY)
    self.pages.append(page)
    self.combined_min = page.min
    self.combined_max = page.max
    self.selected_page = page
    vbox.Add(page, 1, wx.EXPAND)

    page = ToolsPanelIndividual(self, wx.ID_ANY)
    self.pages.append(page)
    self.individual_min = page.min
    self.individual_max = page.max
    self.individual_exp = page.exp
    page.Hide()
    vbox.Add(page, 1, wx.EXPAND)

    self.vbox = vbox
    self.SetSizerAndFit(vbox)

  def SetMode(self, mode):
    self.mode = mode

    if self.selected_page:
      self.selected_page.Hide()

    if mode < len(self.pages):
      page = self.pages[mode]
      page.Show()
    else:
      page = None

    self.selected_page = page

    self.Layout()
    self.Fit()
    self.SetMinSize(self.GetSize())

class ToolsPanelCombined(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    grid = wx.FlexGridSizer(cols=2, vgap=VPAD, hgap=HPAD)

    label = wx.StaticText(self, wx.ID_ANY, 'Min')
    spin = wx.SpinCtrl(self, ID_COMBINED_MIN)
    spin.SetRange(0,32768)
    grid.Add(label)
    grid.Add(spin)
    self.min = spin

    label = wx.StaticText(self, wx.ID_ANY, 'Max')
    spin = wx.SpinCtrl(self, ID_COMBINED_MAX)
    spin.SetRange(0,32768)
    grid.Add(label)
    grid.Add(spin)
    self.max = spin

    self.SetSizerAndFit(grid)

class ToolsPanelIndividual(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    grid = wx.FlexGridSizer(cols=2, vgap=VPAD, hgap=HPAD)

    label = wx.StaticText(self, wx.ID_ANY, 'Min')
    spin = wx.SpinCtrl(self, ID_INDIVIDUAL_MIN)
    spin.SetRange(0,32768)
    grid.Add(label)
    grid.Add(spin)
    self.min = spin

    label = wx.StaticText(self, wx.ID_ANY, 'Max')
    spin = wx.SpinCtrl(self, ID_INDIVIDUAL_MAX)
    spin.SetRange(0,32768)
    grid.Add(label)
    grid.Add(spin)
    self.max = spin

    label = wx.StaticText(self, wx.ID_ANY, 'Exp #')
    spin = wx.SpinCtrl(self, ID_INDIVIDUAL_EXP)
    spin.SetRange(0,0)
    grid.Add(label)
    grid.Add(spin)
    self.exp = spin

    self.SetSizerAndFit(grid)

class ViewModePanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    choices = ['Combined Exposures', 'Individual Exposures', 'Calibration Matrix', 'Processed Spectrum']
    radio = wx.RadioBox(self, ID_VIEW_MODE, 'View', choices=choices,
        majorDimension=1)
    vbox.Add(radio, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.view_mode = radio

    tools = ToolsPanel(self, wx.ID_ANY)
    vbox.Add(tools, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.tools = tools

    self.SetSizerAndFit(vbox)

class FilterPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    label = wx.StaticText(self, wx.ID_ANY, 'Filters:')
    vbox.Add(label, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    filter_list = FilterList(self, wx.ID_ANY, style=wx.BORDER_SUNKEN)
    vbox.Add(filter_list, 1, wx.EXPAND | wx.BOTTOM, VPAD)
    self.filter_list = filter_list

    filter_names = [f[0].name for f in filter_view.REGISTRY]
    filter_choice = wx.Choice(self, wx.ID_ANY, choices=filter_names)
    self.filter_choice = filter_choice
    vbox.Add(filter_choice, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    hbox = wx.BoxSizer(wx.HORIZONTAL)

    button = wx.Button(self, ID_FILTER_ADD, 'Add Filter')
    hbox.Add(button, 1, wx.EXPAND | wx.RIGHT, HPAD)

    button = wx.Button(self, ID_FILTER_REMOVE, 'Remove Filter')
    hbox.Add(button, 1, wx.EXPAND | wx.RIGHT, HPAD)

    vbox.Add(hbox, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    self.Bind(wx.EVT_BUTTON, self.OnButton)

    self.SetSizerAndFit(vbox)

    self.i = 0

  def OnButton(self, evt):
    print ("event ID %s" %evt.Id)
    if evt.Id == ID_FILTER_ADD:
      i = self.filter_choice.GetSelection()
      self.filter_list.AddFilter(i % len(filter_view.REGISTRY))
    elif evt.Id == ID_FILTER_REMOVE:
      self.filter_list.RemoveFilter(0)

class FilterList(ScrolledPanel):
  def __init__(self, *args, **kwargs):
    ScrolledPanel.__init__(self, *args, **kwargs)

    self.grid = wx.FlexGridSizer(cols=4, vgap=VPAD, hgap=HPAD)
    #self.grid.AddGrowableCol(2, 1)
    self.SetSizer(self.grid)

    self.filter_views = []
    self.SetAutoLayout(True)
    self.SetupScrolling(True, True)


  def AddFilter(self, index):
    fltr, view_class, id = filter_view.REGISTRY[index]
    label = wx.StaticText(self, wx.ID_ANY, fltr.name)
    view = view_class(self, id, filter=fltr)

    b = wx.Button(self, wx.ID_ANY, 'X', size=(20, 20))
    sb = wx.SpinButton(self, wx.ID_ANY, style=wx.SP_VERTICAL)

    self.grid.Add(sb, wx.ALIGN_CENTER_VERTICAL)
    self.grid.Add(label, wx.ALIGN_CENTER_VERTICAL)
    self.grid.Add(view, wx.ALIGN_CENTER_VERTICAL)
    self.grid.Add(b, wx.ALIGN_CENTER_VERTICAL)

    self.filter_views.append((b,sb,label,view))

    self.SetupScrolling(False, True)

  def RemoveFilter(self, index):
    for w in self.filter_views[index]:
      self.grid.Remove(w)
      w.Destroy()
    del(self.filter_views[index])
    self.SetupScrolling(False, True)


class ExposureView(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    self.image_view = ImageView(self, ID_EXPOSURE_VIEW, size=(487,195), style=wx.BORDER_SUNKEN)
    vbox.Add(self.image_view, 1, wx.EXPAND)

    self.figure = Figure()
    self.plot = FigureCanvas(self, wx.ID_ANY, self.figure)
    self.plot.SetSize((487,195))
    self.plot.SetWindowStyle(wx.BORDER_SUNKEN)
    vbox.Add(self.plot, 1, wx.EXPAND)
    self.plot.Hide()

    self.toolbar = NavigationToolbar2WxAgg(self.plot)
    vbox.Add(self.toolbar, 0, wx.EXPAND)
    self.plot.Hide()

    self.SetSizerAndFit(vbox)

  def SetViewMode(self, mode):
    if mode == VIEW_MODE_SPECTRUM:
      self.image_view.Hide()
      self.plot.Show()
      self.toolbar.Show()
    else:
      self.image_view.Show()
      self.plot.Hide()
      self.toolbar.Hide()

    self.Layout()

class ExposureSelectorPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    # exposure selector
    label = wx.StaticText(self, wx.ID_ANY, "Exposures:")
    vbox.Add(label, 0, wx.BOTTOM, VPAD)

    listbox = wx.ListBox(self, ID_EXPOSURE_LIST, style=wx.LB_EXTENDED)
    vbox.Add(listbox, 1, wx.EXPAND | wx.BOTTOM, VPAD)
    self.exposure_listbox = listbox

    hbox = wx.BoxSizer(wx.HORIZONTAL)
    button = wx.Button(self, ID_EXPOSURE_ADD, "Add...")
    hbox.Add(button, 0, wx.RIGHT, HPAD)
    button = wx.Button(self, ID_EXPOSURE_DEL, "Delete Selected")
    hbox.Add(button, 0, 0, HPAD)

    vbox.Add(hbox, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    self.SetSizerAndFit(vbox)

class InfoPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)
    grid = wx.FlexGridSizer(3,2, VPAD, HPAD)

    label = wx.StaticText(self, wx.ID_ANY, "Dataset Name:")
    entry = wx.TextCtrl(self, ID_DATASET)
    grid.Add(label, 1, wx.ALIGN_CENTER_VERTICAL)
    grid.Add(entry, 1, wx.EXPAND)
    self.dataset_entry = entry

    label = wx.StaticText(self, wx.ID_ANY, "Incident Energy:")
    entry = wx.TextCtrl(self, ID_ENERGY)
    grid.Add(label, 1, wx.ALIGN_CENTER_VERTICAL)
    grid.Add(entry, 1, wx.EXPAND)
    self.energy_entry = entry

    label = wx.StaticText(self, wx.ID_ANY, "Normalization (I0):")
    entry = wx.TextCtrl(self, ID_NORM)
    grid.Add(label, 1, wx.ALIGN_CENTER_VERTICAL)
    grid.Add(entry, 1, wx.EXPAND)
    self.norm_entry = entry


    label = wx.StaticText(self, wx.ID_ANY, "Spectrometer:")
    choice = wx.Choice(self, ID_SPECTROMETER)
    grid.Add(label, 1, wx.ALIGN_CENTER_VERTICAL)
    grid.Add(choice, 1, wx.EXPAND)
    self.spectrometer_choice = choice

    grid.AddGrowableCol(1, 1)

    vbox.Add(grid, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    # calibration matrix info
    hbox = wx.BoxSizer(wx.HORIZONTAL)
    label = wx.StaticText(self, wx.ID_ANY, "Calibration Matrix:")
    hbox.Add(label, 1, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, HPAD)

    entry = wx.TextCtrl(self, ID_CALIB, style=wx.TE_READONLY)
    entry.SetEditable(False)
    hbox.Add(entry, 2, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, HPAD)
    self.calibration_file_entry = entry

    button = wx.Button(self, ID_CALIB_LOAD, "...")
    hbox.Add(button, 0, 0, HPAD)
    self.load_calibration_button = button

    """
    button = wx.Button(self, ID_CALIB_VIEW, "View")
    hbox.Add(button, 0, 0, HPAD)
    self.view_calibration_button = button
    """

    vbox.Add(hbox, 0, wx.EXPAND | wx.BOTTOM, VPAD)

    self.SetSizerAndFit(vbox)

class ProcessorPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    # first row
    hbox = wx.BoxSizer(wx.HORIZONTAL)

    info = InfoPanel(self, wx.ID_ANY)
    hbox.Add(info, 0, wx.EXPAND | wx.RIGHT, HPAD)

    line = wx.StaticLine(self, wx.ID_ANY, style=wx.LI_VERTICAL)
    hbox.Add(line, 0, wx.EXPAND | wx.RIGHT, HPAD)

    exposure_selector = ExposureSelectorPanel(self, wx.ID_ANY)
    self.exposure_listbox = exposure_selector.exposure_listbox
    hbox.Add(exposure_selector, 2, wx.EXPAND | wx.RIGHT, HPAD)

    vbox.Add(hbox, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    line = wx.StaticLine(self, wx.ID_ANY, style=wx.LI_HORIZONTAL)
    vbox.Add(line, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    
    # second row
    hbox = wx.BoxSizer(wx.HORIZONTAL)

    #filter_panel = FilterPanel(self, wx.ID_ANY)
    #hbox.Add(filter_panel, 1, wx.EXPAND | wx.RIGHT, HPAD)

    exposure_view = ExposureView(self, wx.ID_ANY)

    self.dataset_entry = info.dataset_entry
    self.energy_entry = info.energy_entry
    self.norm_entry = info.norm_entry
    self.spectrometer_choice = info.spectrometer_choice
    self.calibration_file_entry = info.calibration_file_entry

    self.image_view = exposure_view.image_view
    self.plot = exposure_view.plot
    self.exposure_view = exposure_view
    self.figure = exposure_view.figure
    hbox.Add(exposure_view, 1, wx.EXPAND | wx.RIGHT, HPAD)

    mode_panel = ViewModePanel(self, ID_VIEW_MODE)
    hbox.Add(mode_panel, 0, wx.EXPAND | wx.RIGHT, HPAD)
    self.view_mode = mode_panel.view_mode
    self.tools = mode_panel.tools

    vbox.Add(hbox, 2, wx.EXPAND | wx.BOTTOM, VPAD)

    b = wx.Button(self, ID_PROCESS, 'Process Spectrum')
    vbox.Add(b, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.process_button = b


    self.SetSizerAndFit(vbox)


class ProcessorView(MenuFrame):
  def __init__(self, *args, **kwargs):
    kwargs['menu_info'] = [
        ('&File', [
          ('&Open...', wx.ID_OPEN, 'Load Emission Spectrum'),
          ('&Save...', wx.ID_SAVE, 'Save Emission Spectrum'),
          ('', None, None),
          ('E&xit', wx.ID_EXIT, 'Terminate this program'),
          ]),
        ('&Help', [
          ('&About', wx.ID_ABOUT, 'About this program'),
          ])
        ]
    MenuFrame.__init__(self, *args, **kwargs)
    bar = self.CreateStatusBar()
    bar.SetFieldsCount(2)
    bar.SetStatusWidths([-1,-3])

    box = wx.BoxSizer(wx.VERTICAL)
    panel = ProcessorPanel(self, wx.ID_ANY)
    print ("view")
    box.Add(panel, 1, wx.EXPAND | wx.LEFT | wx.TOP, HPAD)

    self.dataset_entry = panel.dataset_entry
    self.energy_entry = panel.energy_entry
    self.norm_entry = panel.norm_entry
    self.calibration_file_entry = panel.calibration_file_entry
    self.spectrometer_choice = panel.spectrometer_choice

    self.exposure_listbox = panel.exposure_listbox
    self.image_view = panel.image_view
    self.view_mode = panel.view_mode
    self.tools = panel.tools
    self.plot = panel.plot
    self.exposure_view = panel.exposure_view
    self.figure = panel.figure
    self.axes = self.figure.add_axes([.1,.1,.8,.8])

    self.SetSizerAndFit(box)

  def SetExposureCount(self, num):
    self.tools.individual_exp.SetRange(1, num)

  def SetIndividualMinMax(self, min, max):
    self.tools.individual_min.SetInt(min)
    self.tools.individual_max.SetInt(max)

  def SetCombinedMinMax(self, min, max):
    self.tools.combined_min.SetInt(min)
    self.tools.combined_max.SetInt(max)

  def SetViewMode(self, mode):
    self.tools.SetMode(mode)
    self.exposure_view.SetViewMode(mode)
    self.view_mode.SetSelection(mode)

  def SetSpectrometer(self, index):
    self.spectrometer_choice.SetSelection(index)

  def SetSpectrometerNames(self, names):
    self.spectrometer_choice.SetItems(names)




WILDCARD_CALIB = "Calibration Files (*.calib)|*.calib|Data Files (*.dat)|*.dat|Text Files (*.txt)|*.txt|All Files|*"
WILDCARD_EXPOSURE = "TIF Files (*.tif)|*.tif|All Files|*"
WILDCARD_XTAL = "All known types|*.xtal;*.calib;*.txt;*.dat|Crystal Files (*.xtal)|*.xtal|Calibration Files (*.calib)|*.calib|Data Files (*.dat)|*.dat|Text Files (*.txt)|*.txt|All Files|*"
WILDCARD_XTAL_EXPORT = "Crystal Files (*.xtal)|*.xtal"
WILDCARD_SCAN = "Scan Files (*.nnnn)|*.????|Text Files (*.txt)|*.txt|All Files|*"
WILDCARD_XES = "XES Data Files (*.xes)|*.xes|Data Files (*.dat)|*.dat|Text Files (*.txt)|*.txt"



import wx
from minixs.gui.frame import MenuFrame
from minixs.gui.image_view import ImageView, ScrolledImageView
from minixs.gui import util

from minixs.gui.calibrator.calibrator_const import *
from matplotlib import cm

from minixs import DIRECTION_NAMES

from minixs import filter, spectrometer
from minixs.gui import filter_view

HPAD = 10
VPAD = 5

class LoadEnergiesPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    self.grid = wx.FlexGridSizer(2, 3, HPAD, VPAD)

    # text file
    label = wx.StaticText(self, wx.ID_ANY, 'Text File:')
    self.grid.Add(label, 0, wx.ALIGN_CENTER_VERTICAL)

    self.file_entry = wx.TextCtrl(self)
    self.grid.Add(self.file_entry, 1, wx.EXPAND)

    b = wx.Button(self, ID_LOAD_SCAN, '...')
    self.grid.Add(b)

    # column
    label = wx.StaticText(self, wx.ID_ANY, 'Column:')
    self.grid.Add(label, 0, wx.ALIGN_CENTER_VERTICAL)

    self.combo = wx.ComboBox(self, wx.ID_ANY, style=wx.CB_READONLY)
    self.grid.Add(self.combo, 1, wx.EXPAND)

    self.grid.AddGrowableCol(1, 1)

    self.SetSizerAndFit(self.grid)

  def fill_column_names(self, filename):
    columns = util.read_scan_column_names(filename)
    sel = self.combo.GetSelection()
    if columns:
      self.combo.SetItems(columns)

      if sel == -1:
        for i in range(len(columns)):
          if 'energy' in columns[i].lower():
            self.combo.SetSelection(i)
            break
      else:
        self.combo.SetSelection(-1) # force update even if sel is same
        self.combo.SetSelection(sel)

  def get_info(self):
    return (self.file_entry.GetValue(), self.combo.GetSelection())

class LoadEnergiesDialog(wx.Dialog):
  def __init__(self, *args, **kwargs):
    wx.Dialog.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    self.panel = LoadEnergiesPanel(self)
    vbox.Add(self.panel, 1, wx.EXPAND | wx.ALL, VPAD)

    hbox = wx.StdDialogButtonSizer()

    b = wx.Button(self, wx.ID_CANCEL, 'Cancel')
    hbox.AddButton(b)

    b = wx.Button(self, wx.ID_OK, 'Add')
    hbox.AddButton(b)

    hbox.Realize()

    vbox.Add(hbox, 0, wx.EXPAND|wx.ALL, VPAD)

    self.SetSizerAndFit(vbox)

  def set_filename(self, filename):
    self.panel.file_entry.SetValue(filename)
    self.panel.fill_column_names(filename)

  def get_info(self):
    return self.panel.get_info()

class FilterPanel(wx.Panel):

  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)
    # The first argument below was increased by one.  This code loops over items in
    # the registry.  After that, one more item is added.  The Sizer does not seem to 
    # automatically allow more items.
    grid = wx.FlexGridSizer(len(filter_view.REGISTRY)+1, 2, HPAD, VPAD)
    self.checks = {}
    self.views = {}

    for fltr, view_class, id in filter_view.REGISTRY:
      check = wx.CheckBox(self, id, fltr.name)
      view = view_class(self, id, filter=fltr)

      enabled, val = self._get_filter_defaults(fltr)

      # set value
      if val is not None:
        view.SetValue(val)

      check.SetValue(enabled)
      view.Enable(enabled)

      grid.Add(check)
      grid.Add(view)

      self.checks[id] = check
      self.views[id] = view

    label = wx.StaticText(self, wx.ID_ANY, 'Dispersive Dir.')
    choice = wx.Choice(self, ID_DISPERSIVE_DIR, choices=DIRECTION_NAMES)
    grid.Add(label, 0, wx.ALIGN_CENTER_VERTICAL)
    grid.Add(choice)
    self.dispersive_direction = choice

    self.SetSizerAndFit(grid)

  def _get_filter_defaults(self, fltr):
    conf = wx.Config.Get()

    # get default value
    valstr = ''
    if hasattr(fltr, 'default_val'):
      valstr = fltr.val_to_str(fltr.default_val)

    # allow configuration to override default
    valstr = conf.Read('calibrator/filters/default_val/%s' % fltr.name, valstr)
    val = fltr.str_to_val(valstr)

    # get default enabled state
    enabled = False
    if hasattr(fltr, 'default_enabled'):
      enabled = fltr.default_enabled

    # allow config to override
    enabled = conf.ReadBool('calibrator/filters/default_enabled/%s' % fltr.name, enabled)

    return (enabled, val)

  def set_filter_value(self, filter_type, value):
    self.views[filter_type].SetValue(int(value))

  def set_filter_enabled(self, filter_type, enabled):
    self.checks[filter_type].SetValue(enabled)
    self.views[filter_type].Enable(enabled)

  def set_filters(self, filters):

    fmap = {}
    for f in filters:
      fmap[f.__class__] = f

    for ftype, view_class, id in filter_view.REGISTRY:
      if ftype in fmap:
        enabled = True
        val = fmap[ftype].get_val()
      else: # only enabled filters are included in `filters`, so for all others must be disabled
        enabled, val = self._get_filter_defaults(ftype)
        enabled = False

      self.checks[id].SetValue(enabled)
      self.views[id].Enable(enabled)
      if val is not None:
        self.views[id].SetValue(val)

  def get_filters(self):
    return [ 
        (
          self.views[i].filter.name,
          self.checks[i].GetValue(),
          self.views[i].GetValue()
        ) for i in filter_view.filter_ids()
      ]

class ExposurePanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    label = wx.StaticText(self, wx.ID_ANY, 'No Exposures Loaded')
    vbox.Add(label, 0, wx.EXPAND, VPAD)
    self.label = label

    #panel = ScrolledImageView(self, ID_IMAGE_PANEL, size=(487,195))
    #self.image_view = panel.image_view
    panel = ImageView(self, ID_IMAGE_PANEL, size=(487,195))
#     panel = ImageView(self, ID_IMAGE_PANEL, size=(1475,1679))
    self.image_view = panel
    vbox.Add(panel, 1, wx.EXPAND | wx.BOTTOM |wx.RIGHT, VPAD)

    slider = wx.Slider(self, ID_EXPOSURE_SLIDER, 0,0,1)
    slider.Enable(False)
    vbox.Add(slider, 0, wx.EXPAND )
    self.slider = slider

    self.SetSizerAndFit(vbox)

  def SetPixels(self, pixels, colormap=cm.Greys_r):
    self.image_view.SetPixels(pixels, colormap)


import wx.lib.mixins.listctrl as listmix
class ExposureList(wx.ListCtrl,
                   listmix.ListCtrlAutoWidthMixin,
                   listmix.TextEditMixin
                   ):
  def __init__(self, *args, **kwargs):
    wx.ListCtrl.__init__(self, *args, **kwargs)
    listmix.ListCtrlAutoWidthMixin.__init__(self)
    listmix.TextEditMixin.__init__(self)

    self.InsertColumn(0, 'Incident Energy', width=200)
    self.InsertColumn(1, 'Exposure File', width=200)

    self.num_energies = 0
    self.num_exposures = 0

  def FindLastEmptyItem(self, column):
    count = self.GetItemCount()
    # find last empty energy
    i = count - 1
    while i >= 0:
      text = self.GetItem(i,column).GetText()
      if text != '':
        break

      i -= 1
    index = i+1

    return index

  def AppendEnergy(self, energy):
    s = '%.2f' % energy

    count = self.GetItemCount()
    index = self.FindLastEmptyItem(0)

    if index >= count:
      self.InsertStringItem(index, '')

    self.SetStringItem(index, 0, s)
    self.EnsureVisible(index)

  def AppendExposure(self, exposure):
    count = self.GetItemCount()
    index = self.FindLastEmptyItem(1)

    if index >= count:
      self.InsertStringItem(index, '')

    self.SetStringItem(index, 1, exposure)
    self.EnsureVisible(index)

  def AppendRow(self):
    """Add empty row to end of listctrl"""
    count = self.GetItemCount()
    self.InsertStringItem(count, '')
    self.EnsureVisible(count)

  def DeleteRow(self, index=None):
    if index is None and self.GetItemCount() == 1:
      self.DeleteItem(0)

    if index is None:
      index = self.GetFirstSelected()
      while index != -1:
        self.DeleteItem(index)
        index = self.GetFirstSelected()
    else:
      self.DeleteItem(index)

  def ClearEnergies(self):
    for i in range(self.GetItemCount()):
      self.SetStringItem(i, 0, '')
    self.num_energies = 0

  def ClearExposures(self):
    for i in range(self.GetItemCount()):
      self.SetStringItem(i, 1, '')
    self.num_exposures = 0

  def GetData(self):
    nr = self.GetItemCount()
    nc = self.GetColumnCount()
    strings = [
        [self.GetItem(i,j).GetText() for j in range(nc)]
        for i in range(nr)
        ]

    energies = []
    files = []
    valid = True
    for row in strings:
      e,f = row
      if e == '' and f == '':
        continue
      if e == '' or f == '':
        valid = False
        continue
      energies.append(float(e))
      files.append(f)

    return (valid, energies, files)

class ToolsPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)

    vbox = wx.BoxSizer(wx.VERTICAL)

    choices = ['Exposures', 'Calibration Matrix']
    radio = wx.RadioBox(self, ID_VIEW_TYPE, 'View', choices=choices,
        majorDimension=1)
    vbox.Add(radio, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.view_type = radio

    button = wx.Button(self, ID_FIND_XTALS, 'Find Crystal Bounds')
    vbox.Add(button)

    check = wx.CheckBox(self, ID_SHOW_XTALS, 'Show Crystals')
    check.SetValue(True)
    vbox.Add(check)

    self.SetSizerAndFit(vbox)

class CalibratorPanel(wx.Panel):
  def __init__(self, *args, **kwargs):
    wx.Panel.__init__(self, *args, **kwargs)
    self.splitter = wx.SplitterWindow(self)
    topPanel = wx.Panel(self.splitter)

    vboxTop = wx.BoxSizer(wx.VERTICAL)

    # spectrometer
    """
    hbox = wx.BoxSizer(wx.HORIZONTAL)
    label = wx.StaticText(self, wx.ID_ANY, "Spectrometer: ")
    self.spectrometer_tags, spectrometer_names = spectrometer.list_spectrometers(include_names=True)
    spectrometer_names.append('Other')
    choice = wx.Choice(self, ID_SPECTROMETER, choices=spectrometer_names)
    hbox.Add(label, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, HPAD)
    hbox.Add(choice, 1, wx.RIGHT, HPAD)
    vboxTop.Add(hbox, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.spectrometer_choice = choice
    """

    # dataset name box
    hbox = wx.BoxSizer(wx.HORIZONTAL)
    label = wx.StaticText(topPanel, wx.ID_ANY, "Dataset Name: ")
    entry = wx.TextCtrl(topPanel, ID_DATASET_NAME)
    hbox.Add(label, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, HPAD)
    hbox.Add(entry, 1, wx.RIGHT, HPAD)
    vboxTop.Add(hbox, 0, wx.EXPAND | wx.BOTTOM, VPAD)
    self.dataset_name = entry


    # exposure list and control buttons
    hbox = wx.BoxSizer(wx.HORIZONTAL)

    # exposures list
    listctrl = ExposureList(topPanel, ID_EXPOSURE_LIST,
        style=wx.LC_REPORT|wx.LC_HRULES|wx.LC_VRULES,
        size=(200,200))
    hbox.Add(listctrl, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, HPAD)
    self.exposure_list = listctrl

    vbox2= wx.BoxSizer(wx.VERTICAL)

    button = wx.Button(topPanel, ID_READ_ENERGIES, "Read Energies...")
    vbox2.Add(button, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    button = wx.Button(topPanel, ID_SELECT_EXPOSURES, "Select Exposures...")
    vbox2.Add(button, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    button = wx.Button(topPanel, ID_APPEND_ROW, "Append Row")
    vbox2.Add(button, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    button = wx.Button(topPanel, ID_DELETE_ROW, "Delete Row(s)")
    vbox2.Add(button, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    button = wx.Button(topPanel, ID_CLEAR_ENERGIES, "Clear Energies")
    vbox2.Add(button, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    button = wx.Button(topPanel, ID_CLEAR_EXPOSURES, "Clear Exposures")
    vbox2.Add(button, 1, wx.EXPAND)

    hbox.Add(vbox2, 0, wx.EXPAND | wx.RIGHT, HPAD)

    vboxTop.Add(hbox, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    topPanel.SetSizerAndFit(vboxTop)

    # add filters and image view
    bottomPanel = wx.Panel(self.splitter)
    vboxBottom = wx.BoxSizer(wx.VERTICAL)
    hboxBottom = wx.BoxSizer(wx.HORIZONTAL)
    
    panel = FilterPanel(bottomPanel, wx.ID_ANY)
    hboxBottom.Add(panel, 0, wx.LEFT | wx.RIGHT, HPAD)
    self.filter_panel = panel

    panel = ExposurePanel(bottomPanel, wx.ID_ANY)
    hboxBottom.Add(panel, 1, wx.EXPAND | wx.RIGHT | wx.BOTTOM, HPAD)
    self.exposure_panel = panel

    panel = ToolsPanel(bottomPanel, wx.ID_ANY)
    hboxBottom.Add(panel, 0, wx.RIGHT, HPAD)
    self.tools_panel = panel

    vboxBottom.Add(hboxBottom, 1, wx.EXPAND | wx.BOTTOM, VPAD)

    # load calibrate button
    button = wx.Button(bottomPanel, ID_CALIBRATE, "Calibrate")
    vboxBottom.Add(button, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, HPAD)
    self.calibrate_button = button

    bottomPanel.SetSizerAndFit(vboxBottom)
    self.splitter.SplitHorizontally(topPanel, bottomPanel)
    sizerWindow = wx.BoxSizer(wx.VERTICAL)
    sizerWindow.Add(self.splitter,1, wx.EXPAND)
    self.SetSizer(sizerWindow)
    sizerWindow.Fit(self)
    self.Layout()
    
class CalibratorView(MenuFrame):
  def __init__(self, *args, **kwargs):
    kwargs['menu_info'] = [
        ('&File', [
          ('&Open...', wx.ID_OPEN, 'Load Calibration'),
          ('&Save...', wx.ID_SAVE, 'Save Calibration'),
          ('', None, None), # separator
          ('&Import Crystals...', ID_IMPORT_XTALS, 'Import Crystals'),
          ('&Export Crystals...', ID_EXPORT_XTALS, 'Export Crystals'),
          ('', None, None), # separator
          ('E&xit', wx.ID_EXIT, 'Terminate this program'),
          ]),
        ('&Help', [
          ('&About', wx.ID_ABOUT, 'About this program')
          ]),
        ]
    MenuFrame.__init__(self, *args, **kwargs)
    bar = self.CreateStatusBar()
    bar.SetFieldsCount(2)
    bar.SetStatusWidths([-1, -3])

    box = wx.BoxSizer(wx.VERTICAL)
    self.panel = CalibratorPanel(self, wx.ID_ANY)
    box.Add(self.panel, 1, wx.EXPAND)

    self.SetSizerAndFit(box)

    # provide shortcuts for gui elements that controller needs access to
    self.image_view = self.panel.exposure_panel.image_view
    self.exposure_list = self.panel.exposure_list
    self.dataset_name = self.panel.dataset_name
    self.exposure_panel = self.panel.exposure_panel
    self.exposure_label = self.exposure_panel.label
    self.exposure_slider = self.exposure_panel.slider
    self.view_type = self.panel.tools_panel.view_type
    self.calibrate_button = self.panel.calibrate_button

  def get_filters(self):
    return self.panel.filter_panel.get_filters()




from minixs.scanfile import ScanFile

def read_scan_column_names(scanfile_name):
#   with open(scanfile) as f:
#     last = None
# 
#     for line in f:
#       if line[0] != "#":
#         # get number of columns in first data line
#         num = len(line.split())
# 
#         if last is None:
#           num = len(line.split())
#           return [ str(i) for i in range(1,num+1) ]
# 
#         else:
#           last = last[1:]
#           cols = last.split()
# 
#           # if whitespace separated headers aren't correct, try fixed width
#           # XXX this should be done better by actually detecting a PNC file
#           if len(cols) != num:
#             w = 21
#             cols = [ last[w*i:w*(i+1)].strip() for i in range(0,len(last)/20) ]
#           if len(cols) != num:
#             return [ str(i) for i in range(1,num+1) ]
# 
#           return [ "%d: %s" % (i,s) for i,s in zip(range(1,num+1), cols) ]
# 
#       last = line
    s = ScanFile(scanfile_name)
    num_cols = len(s.columns)
    return [ "%d: %s" % (i,s) for i,s in zip(range(1,num_cols+1), s.columns) ]
    
def xtals_intersect(xtal1, xtal2):
  (ax1, ay1), (ax2, ay2)  = xtal1
  (bx1, by1), (bx2, by2)  = xtal2

  return (ax1 < bx2 and bx1 < ax2 and 
      ay1 < by2 and by1 < ay2)
  



'''
 Copyright (c) 2019, UChicago Argonne, LLC
 See LICENSE file.
'''
from PyQt5.QtCore import QAbstractTableModel, QModelIndex
from PyQt5.QtCore import Qt
from minixs.calibrate import Calibration
import logging
from minixs import METHOD_ENTER_STR
from PyQt5.Qt import QVariant
logger = logging.getLogger(__name__)

TABLE_DATA_OUT_OF_RANGE = "Trying to access data that is out of range"
ENERGY_HEADER = "Incident Energy"
EXPOSURE_HEADER = "Exposure File"
ENERGY_COLUMN_NUMBER = 0
EXPOSURE_COLUMN_NUMBER = 1
TOTAL_COLUMNS = 2

class ExposureListModel(QAbstractTableModel):
    '''
    Model for holding information about the Exposures to be used.  Model part
    of Model/View pattern
    '''


    def __init__(self, *args, **kwargs):
        '''
        Constructor
        '''
        super(ExposureListModel, self).__init__(*args, **kwargs)
        self.modelData = Calibration()
        
    def appendEnergy(self, energy):
        '''
        Append energy value the table.  New energy values are appended
        after the last non empty space in the table.  If there is no 
        space, the list is expanded.
        '''
        count = self.rowCount()
        index = self.findLastEmptyItem(ENERGY_COLUMN_NUMBER)
        
        if index >= count:
            self.insertRow(index, )
        self.setData(self.index(index, ENERGY_COLUMN_NUMBER, self.parent), energy)
        
        
    def appendExposure(self, exposure_file):
        '''
        Append Exposure value to the table.  New values are appended
        after the last non empty space.  If there is no space, the 
        list is expanded.
        '''
        count = self.rowCount()
        index = self.findLastEmptyItem(EXPOSURE_COLUMN_NUMBER)
        
        if index >= count:
            self.insertRow(index, )
        self.setData(self.index(index, EXPOSURE_COLUMN_NUMBER, self.parent), exposure_file)
    
    def columnCount(self, parent=QModelIndex()):
        '''
        Provide column count.  Always 2 Energy and Exposure File
        '''
        return TOTAL_COLUMNS
    
    def data(self, index, role=Qt.DisplayRole):
        '''
        Method to retrieve data from the model.  Data is retrieved by
        index and by role.
        '''
        logger.debug(METHOD_ENTER_STR % ((index.row(), index.column(), role),))
        if not( role == Qt.DisplayRole or role == Qt.EditRole or 
                role == Qt.StatusTipRole):
            return QVariant()
        if index.row() >= self.rowCount():
            logger.exception("Index.row %s >= rowCount %s" % (index.row(), self.rowCount()))
            raise ValueError("%s %s" % \
                             (TABLE_DATA_OUT_OF_RANGE, \
                              (index.row(), index.column)))
        if not index.isValid():
            logger.error("Index not valid")
            return QVariant()
        if role == Qt.DisplayRole or Qt.EditRole:
            if index.column() == ENERGY_COLUMN_NUMBER:
                logger.debug("Reading data for energy(%s) %s" % (index.row(), self.modelData.energies[index.row()]))
                logger.debug("self.modelData.energies %s" % self.modelData.energies)
                return self.tr(str(self.modelData.energies[index.row()]))
            elif index.column() == EXPOSURE_COLUMN_NUMBER:
                logger.debug("Reading data for exposure_files(%s) %s" % (index.row(), self.modelData.exposure_files[index.row()]))
                return self.tr(self.modelData.exposure_files[index.row()])
            else:
                logger.exception("wrong column(%s) for row %s" % (index.row(), index.column()))
                raise ValueError("%s %s" %
                                 (TABLE_DATA_OUT_OF_RANGE, 
                                  (index.row(), index.column)))
        if role == Qt.StatusTipRole:
            return self.tr("Energy: %s, Filename: %s" % 
                           (self.modelData.energies[index.row()], 
                            self.modelData.exposure_files[index.row()]))
    
    def index(self, row, column, parent):
        return self.createIndex(row, column)
    
    def findLastEmptyItem(self, column):
        logger.debug(METHOD_ENTER_STR)
        count = self.rowCount()
        i = count-1;
        logger.debug("rowCount %s" % count)
        while i>=0:
            if column == ENERGY_COLUMN_NUMBER:
                logger.debug("i %s, self.model.energies: %s" %(i, self.modelData.energies))
                if self.modelData.energies[i] != None:
                    break
                i -= 1
            elif column == EXPOSURE_COLUMN_NUMBER:
                if self.modelData.exposure_files[i] != None:
                    break
                i -= 1
        index = i + 1
        logger.debug("Found last item at index %s" % index)
        return index

    def flags(self, index):
        return Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
    
    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role != Qt.DisplayRole:
            return QVariant()
        
        if orientation == Qt.Horizontal:
            if section == ENERGY_COLUMN_NUMBER:
                logger.debug("Getting section %s, %s role %s" % (section, ENERGY_HEADER, role))
                return self.tr(ENERGY_HEADER)
            elif section == EXPOSURE_COLUMN_NUMBER:
                logger.debug("Getting section %s, %s role %s" % (section, EXPOSURE_HEADER, role))
                return self.tr(EXPOSURE_HEADER)
        else: 
            return self.tr(str(section))
        
    def insertRows(self, row, count, parent=QModelIndex()):
        logger.debug(METHOD_ENTER_STR % ((row, count),))
        self.beginInsertRows(parent, row, row+count-1)
        for index in range(count):
            self.modelData.energies.insert(row+index, None)
            self.modelData.exposure_files.insert(row+index, None)
        self.endInsertRows()
        return True
        
    def parent(self, index):
        return self.parent
    
    def removeRows(self, row, count, parent=QModelIndex()):
        logger.debug(METHOD_ENTER_STR % ((row, count),))
        self.beginRemoveRows(parent, row, row+count-1)
        for index in range(count):
            self.modelData.energies.remove(self.modelData.energies[index+row])
            self.modelData.exposure_files.remove(\
                                    self.modelData.exposure_files[index+row])
        self.endRemoveRows()
        return True
        
        
    def setData(self, index, value, role=Qt.EditRole):
        logger.debug(METHOD_ENTER_STR % ((index.row(),index.column(), value, role),))
        if index.column() == ENERGY_COLUMN_NUMBER:
            self.modelData.energies[index.row()] = value
            logger.debug("Changing data for energy(%s) %s" %(index.row(), self.modelData.energies[index.row()]))
        elif index.column() == EXPOSURE_COLUMN_NUMBER:
            self.modelData.exposure_files[index.row()] = value
            logger.debug("Changing data for exposure_files(%s) %s" %(index.row(), self.modelData.exposure_files[index.row()]))
        logger.debug("self.modelData.energies %s" % self.modelData.energies)
        self.dataChanged.emit(index, index)
        return True
    
    def rowCount(self, parent=QModelIndex()):
        return max(len(self.modelData.energies), 
                   len(self.modelData.exposure_files))

    
    def setCalibration(self, calibrationModel):
        self.modelData = calibrationModel


import wx
import view, controller
from const import *

from minixs import filter, emission
from minixs.gui import filter_view

class ProcessorModel(emission.EmissionSpectrum):
  def validate(self):
    self.validation_errors = []
    if len(self.exposure_files) < 1:
      self.validation_errors.append("No exposure files have been selected.")
    if not self.calibration_file:
      self.validation_errors.append("No calibration matrix has been specified.")

    return len(self.validation_errors) == 0

class ProcessorApp(wx.App):
  def __init__(self, *args, **kwargs):
    wx.App.__init__(self, *args, **kwargs)
    self.SetAppName('minixs')

    v = view.ProcessorView(None, ID_MAIN_FRAME, 'miniXS XES Processor')
    m = ProcessorModel()
    c = controller.ProcessorController(v, m)
    
    v.Show()

def main():

  for f in filter.REGISTRY:
    if hasattr(f, 'view_name'):
      view_class = getattr(filter_view, f.view_name)
      filter_view.register(f, view_class)

  app =  ProcessorApp()
  app.MainLoop()

if __name__ == "__main__":
  main()



"""
Exposure filters

A set of filters that act on exposure pixel arrays.

See the `Filter` class documentation for details on how to implement new
filters.
"""

import numpy as np
import logging
from minixs import METHOD_ENTER_STR
logger = logging.getLogger(__name__)

MIN_VISIBLE_FILTER = "Min Visible"
MAX_VISIBLE_FILTER = "Max Visible"
LOW_CUTOFF_FILTER = "Low Cutoff"
HIGH_CUTOFF_FILTER = "High Cutoff"
NEIGHBORS_FILTER = "Neighbors"
BAD_PIXELS_FILTER = "Bad Pixels"
EMISSION_FILTER = "Emission Filter"

#########################
#                       #
# Abstract Base Classes #
#                       #
#########################
class Filter(object):
  """
  Abstract filter base class

  Filters take an array of pixels and process them in some fashion

  To implement a new filter type, create a subclass of Filter.

  Set the name, default_val and default_enabled class variables.

  Implement class methods `str_to_val` and `val_to_str` that convert the
  filter's parameters between a string representation (to be saved in an
  output file) and the internal representation (of any type).

  Finally, implement the object method `filter`, which applies this filter to
  an array of pixels (in place).

  To create a filter that has only one parameter of a basic type, subclass
  one of IntFilter, FloatFilter, or StringFilter. These already have
  implementations of `str_to_val` and `val_to_str`.
  """
  name = ""
  default_val = None
  default_enabled = False

  def __init__(self, val=None):
    self.enabled = True
    if val is None:
      self.val = self.default_val
    else:
      self.set_val(val)

    self.region = None

  def filter(self, pixels, energy):
    """
    Apply this filter to the set regions of an array of pixels.
    This calls `filter_region  for the portion of `pixels` that
    was set using `set_region`.

    Subclass should overrid the `filter_region` method.
    """
    if self.region is None:
      self.filter_region(pixels, energy)
    else:
      x1,y1,x2,y2 = self.region
      self.filter_region(pixels[y1:y2,x1:x2], energy)

  def filter_region(self, pixels, energy):
    """
    Apply this filter to an array of pixels

    This must be overridden by subclasses.

    Parameters
    ----------
      pixels: an array of counts received at each pixel
      energy: the incident beam energy
    """
    raise ValueError("Unimplemented Filter: %s" % self.name)

  def set_str(self, valstr):
    """
    Set filter's value from a string
    """
    self.val = self.str_to_val(valstr)

  def get_str(self):
    """
    Get filter's value as a string
    """
    return self.val_to_str(self.val)

  def set_val(self, val):
    """
    Set filter's value
    """
    self.val = val

  def get_val(self):
    """
    Get filter's value
    """
    return self.val

  def set_region(self, region):
    """
    Set a rectangular region to limit this filter to
    """
    if not self.region_allowed:
      raise Exception("The '%s' filter does not allow setting a region" % self.name)

    self.region = region

  @classmethod
  def str_to_val(cls,valstr):
    """
    Convert value from string to whichever representation is used internally

    This is used to load a filter from a file
    """
    return None

  @classmethod
  def val_to_str(cls,val):
    """
    Convert value from internal representation to string

    This is used to save a filter to a file
    """
    return str(val)

class IntFilter(Filter):
  """
  An abstract class for a filter that takes a single integer parameter
  """
  view_name = "IntFilterView"

  @classmethod
  def str_to_val(cls,valstr):
    if valstr == '':
      return 0
    else:
      return int(valstr)

class FloatFilter(Filter):
  """
  An abstract class for a filter that takes a single float parameter
  """
  view_name = "FloatFilterView"

  @classmethod
  def val_to_str(cls,val):
    return "%.2f" % val

  @classmethod
  def str_to_val(cls,valstr):
    if valstr == '':
      return 0
    else:
      return float(valstr)

class StringFilter(Filter):
  """
  An abstract class for a filter that takes a single string parameter
  """
  view_name = "StringFilterView"

  @classmethod
  def val_to_str(cls,val):
    return val

class ChoiceFilter(Filter):
  """
  An abstract class for a filter that has a single parameter chosen from a set

  Subclasses should override the `CHOICES` class variable with a list of
  strings values of possible choices.

  The `val` object variable will be set to the index of the selected choice.
  """
  view_name = "ChoiceFilterView"
  default_value = 0

  CHOICES = [
      ]

  @classmethod
  def val_to_str(cls,val):
    if val == -1:
      return ''

    try:
      return cls.CHOICES[val]
    except IndexError:
      return ''

  @classmethod
  def str_to_val(cls,valstr):
    if valstr == '':
      return -1

    return cls.CHOICES.index(valstr)

##################
#                #
# Actual Filters #
#                #
##################

class MinFilter(IntFilter):
  """
  A dummy filter used by the Calibrator GUI to set the lower contrast level to show
  """
  name = MIN_VISIBLE_FILTER
  view_name = "IntFilterView"
  default_val = 0
  default_enabled = True
  region_allowed = False

  def filter_region(self, pixels, energy):
    pass

class MaxFilter(IntFilter):
  """
  A dummy filter used by the Calibrator GUI to set the upper contrast level to show
  """
  name = MAX_VISIBLE_FILTER
  view_name = "IntFilterView"
  default_val = 1000
  default_enabled = False
  region_allowed = False

  def filter_region(self, pixels, energy):
    pass

class LowFilter(IntFilter):
  """
  Zero out all pixels below a given threshold
  """
  name = LOW_CUTOFF_FILTER
  default_val = 5
  default_enabled = True
  region_allowed = True

  def filter_region(self, pixels, energy):
    logger.debug(METHOD_ENTER_STR, ((self.name, pixels, energy,self.val)))
    if self.val is None:
      return
    pixels[np.where(pixels < self.val)] = 0

class HighFilter(IntFilter):
  """
  Zero out all pixels above a given threshold
  """
  name = HIGH_CUTOFF_FILTER
  default_val = 1000
  default_enabled = False
  region_allowed = True

  def filter_region(self, pixels, energy):
    logger.debug(METHOD_ENTER_STR, ((self.name, pixels, energy, self.val)))
    if self.val is None:
      return
    pixels[np.where(pixels > self.val)] = 0

class NeighborFilter(IntFilter):
  """
  Zero out all pixels with fewer than the given number of nonzero neighbors

  A neighbor is defined as a pixel whose row and column are within one of the
  pixel under consideration, not including the pixel itself. Each pixel has 8
  neighbors.
  """
  name = NEIGHBORS_FILTER
  default_val = 2
  default_enabled = True
  region_allowed = True

  def filter_region(self, pixels, energy):
    logger.debug(METHOD_ENTER_STR, ((self.name, pixels, energy,self.val)))
    from itertools import product
    nbors = (-1,0,1)
    mask = np.sum([
      np.roll(np.roll(pixels > 0,i,0),j,1)
      for i,j in product(nbors,nbors)
      if i != 0 or j != 0
      ], 0) >= self.val 
    pixels *= mask

class BadPixelFilter(Filter):
  """
  Filter out a set of specified pixels

  Each pixel in the provided list is processed in manner which depends on the
  mode:

    0: Zero out pixels
    1: Linearly interpolate value from pixels to left and right
    2: Linearly interpolate value from pixels above and below
  """

  name = BAD_PIXELS_FILTER

  default_val = (0,[])
  default_enabled = False
  region_allowed = False

  MODE_ZERO_OUT = 0
  MODE_INTERP_H = 1
  MODE_INTERP_V = 2

  @classmethod
  def str_to_val(cls, valstr):
    tmp = valstr.split('|')
    if len(tmp) == 2:
      mode = int(tmp[0])
      pts = tmp[1].split(';')
      bad_pixels = [[int(c.strip()) for c in pt.split(',')] for pt in pts]
      return (mode, bad_pixels)
    else:
      raise Exception("Invalid filter value: %s" % valstr)

  @classmethod
  def val_to_str(cls, val):
    mode, points = val
    points = ';'.join(['%d,%d' % tuple(p) for p in points])
    return '%d|%s' % (mode, points)

  def filter(self, pixels, energy):
    mode, bad_pixels = self.val
    for x,y in bad_pixels:
      if mode == self.MODE_ZERO_OUT:
        pixels[y,x] = 0
      elif mode == self.MODE_INTERP_H:
        pixels[y,x] = (pixels[y,x-1] + pixels[y,x+1])/2.
      elif mode == self.MODE_INTERP_V:
        pixels[y,x] = (pixels[y-1,x] + pixels[y+1,x])/2.

class EmissionFilter(ChoiceFilter):
  """
  A filter to remove fluorescence from elastic exposures

  This is hard coded to only handle the Fe Kbeta von Hamos
  spectrometer. At some point, this should be replaced
  by something more flexible.
  """
  name = EMISSION_FILTER

  TYPE_FE_KBETA = 0
  TYPE_FE_KBETA_TIGHT = 1
  CHOICES = [
      "Fe Kbeta",
      "Fe Kbeta v2"
      ]

  default_val = TYPE_FE_KBETA
  default_enabled = False
  region_allowed = False

  def filter(self, pixels, energy):
    if self.val == self.TYPE_FE_KBETA:
      if energy >= 7090:
        z = 1.3214 * energy - 9235.82 - 12
        pixels[0:z,:] = 0
    elif self.val == self.TYPE_FE_KBETA_TIGHT:
      m = [ 1.267343, 1.265995, 1.267343, 1.275763, 1.258217, 1.254672, 1.254672, 1.267664, 1.304953, 1.289443 ]
      b = [ -8847.35, -8846.56, -8849.35, -8913.20, -8790.86, -8762.06, -8761.06, -8847.43, -9109.12, -9006.66 ]
      w = pixels.shape[1]

      if energy >= 7090:
        for i in range(10):
          x1 = np.floor(w / 10.0 * i)
          x2 = np.ceil(w / 10.0 * (i+1))
          z = m[i] * energy + b[i] - 1
          pixels[0:z,x1:x2] = 0
    else:
      raise ValueError("Unimplemented Emission Filter")


###################
#                 #
# Filter Registry #
#                 #
###################

REGISTRY = []
FILTER_MAP = {}


def get_filter_by_name(name):
  """
  Lookup filter by name

  Parameters
  ----------
    name: the name of the filter

  Returns
  -------
    an instantiated filter object of the given type or None if name is unknown
  """

  global FILTER_MAP
  fltr = FILTER_MAP.get(name, None)
  if fltr is None:
    return None
  else:
    return fltr()

def register(f):
  """
  Register a filter

  Parameters
  ----------
    f: a class derived from Filter
  """
  FILTER_MAP[f.name] = f
  REGISTRY.append(f)
  

# register standard filters
FILTERS = [
  MinFilter,
  MaxFilter,
  LowFilter,
  HighFilter,
  NeighborFilter,
  BadPixelFilter,
  EmissionFilter
  ]
for f in FILTERS:
  register(f)



"""
XES Spectrum processing code
"""

import os, sys
import numpy as np

import minixs.calibrate
from minixs.exposure import Exposure
from minixs.misc import read_scan_info, gen_file_list
from minixs.constants import *
from minixs.filter import  get_filter_by_name
from minixs.parser import Parser, STRING, INT, FLOAT, LIST
from minixs import METHOD_ENTER_STR
try:
    from itertools import izip as zip     # Python 2 way, from Python 3 can use zip directly
except ImportError:
    pass                      #Python 3 do nothing
import logging
logger = logging.getLogger(__name__)

def load(filename):
  """Load EmissionSpectrum from file"""
  xes = EmissionSpectrum()
  xes.load(filename)
  return xes

def process_spectrum(cal, exposure, energies, I0, direction, xtals, solid_angle=None, skip_columns=[]):
  """Interpolated emission spectrum

  Parameters
  ----------
  cal : calibration matrix
  exposure : spectrum Exposure
  energies : list of emission energies for desired spectrum
  I0 : intensity normalization value
  direction: dispersive direction (minixs.HORIZONTAL or minixs.VERTICAL)
  xtals: list of crystal rects
  solid_angle: an array giving the solid angle subtended by each pixel

  If solid_angle is not given, then it is effectively an array of ones
  """
  logger.debug(METHOD_ENTER_STR % ((cal, exposure, energies, I0, direction, xtals),))
  intensity = np.zeros(energies.shape)
  mask = np.zeros(energies.shape)
  num_pixels = np.zeros(energies.shape)

  #variance = np.zeros(energies.shape)
  #y_i = np.zeros(energies.shape)
  #var_i = np.zeros(energies.shape)
  logger.debug("cal shape %s,exposure shape %s" %(cal.shape, exposure.pixels.shape))
  for xtal in xtals:
    (x1,y1), (x2,y2) = xtal

    if direction == DOWN or direction == UP:
      i1, i2 = x1, x2
    else:
      i1, i2 = y1, y2

    for i in range(i1,i2):
      if i in skip_columns:
        #sys.stderr.write("skipping %d\n" % i)
        continue

      # select one row/column of calibration matrix and correspond row/column of spectrum exposure
      # XXX this is untested for UP and RIGHT, but *should* be correct
      if direction == UP:
        dE = cal[y1:y2,i]
        dI = exposure.pixels[y1:y2,i]
        if solid_angle is not None:
          dS = solid_angle[y1:y2,i]
      elif direction == DOWN:
        dE = cal[y2-1:y1-1:-1,i]
        dI = exposure.pixels[y2-1:y1-1:-1,i]
        if solid_angle is not None:
          dS = solid_angle[y2-1:y1-1:-1,i]
      elif direction == RIGHT:
        dE = cal[i, x1:x2]
        dI = exposure.pixels[i, x1:x2]
        if solid_angle is not None:
          dS = solid_angle[i, x1:x2]
      elif direction == LEFT:
        dE = cal[i, x2-1:x1-1:-1]
        dI = exposure.pixels[i, x2-1:x1-1:-1]
        if solid_angle is not None:
          dS = solid_angle[i, x2-1:x1-1:-1]
      else:
        raise Exception("Invalid direction.")

      # XXX: the following uses a quick method that overestimates statistical error
      #      to correctly propogate error, use interp_poisson() (which is much slower at the moment)
      #      this should be made optional so that one can do quick processing at the beamline
      #      and then get correct errorbars later
      #      (i should also characterize how incorrect the errors are...)
      #      example: interp_poisson(energies, y_i, var_i, dE, dI,-1,-1)
      y_i = np.interp(energies, dE, dI,-1,-1)
      mask *= 0
      mask[np.where(y_i >= 0)] = 1
      intensity += y_i * mask
      if solid_angle is not None:
        s_i = np.interp(energies, dE, dS, -1,-1)
        num_pixels += mask * s_i
      else:
        num_pixels += mask

      #variance += var_i * mask

  # if num_pixels is 0, then intensity will also be 0, so divide by 1 instead of 0 to avoid NaN
  norm = num_pixels.copy()
  norm[np.where(norm == 0)] = 1

  # create N x 5 array with columns: energy, normalized intensity, uncertainty, raw counts, number of rows/columns contributing
  return np.vstack([energies, intensity/I0/norm, np.sqrt(intensity)/I0/norm, intensity, num_pixels]).T

def binned_emission_spectrum(calib, exposure, low_energy, high_energy, energy_step, I0):
  """
  Calculate emission spectrum from exposure and calibration matrix

  Return:
    array of (emission_energy, I, sigma, raw_counts, num_pixels)
      I = raw_counts / num_pixels / I0
      sigma = sqrt(raw_counts) / num_pixels / I0

  """

  if low_energy is None:
    low_energy = calib[np.where(calib > 0)].min()
  if high_energy is None:
    high_energy = calib.max()

  energies = np.arange(low_energy, high_energy, energy_step)
  spectrum = np.zeros((len(energies), 5))
  spectrum[:,0] = energies + energy_step / 2.
  for E,counts in zip(calib.flat,exposure.pixels.flat):
    binnum = (E - low_energy) / energy_step

    # skip pixels outside of desired range
    if binnum < 0 or binnum >= len(energies):
      continue

    spectrum[binnum][3] += counts
    spectrum[binnum][4] += 1

  # calculate normalized intensity and uncertainty
  i = np.where(spectrum[:,4] > 0)
  spectrum[i,1] = (spectrum[i,4]>0) * spectrum[i,3] / spectrum[i,4] / I0
  spectrum[i,2] = (spectrum[i,4]>0) * np.sqrt(spectrum[i,3]) / spectrum[i,4] / I0

  return spectrum

def process_all(calibfile, scanfile, base_image, image_nums, E_column=0, I0_column=6, low_cutoff=0, high_cutoff=1000, low_energy=None, high_energy=None, energy_step = 0.5, zero_pad=3):

  calib = np.loadtxt(calibfile)

  if low_energy is None:
    low_energy = calib[np.where(calib > 0)].min()

  if high_energy is None:
    high_energy = calib.max()


  Es, I0s = read_scan_info(scanfile, [E_column, I0_column])
  filenames = gen_file_list(base_image, image_nums, zero_pad)

  print ("Directory: %s" % os.path.dirname(filenames[0]))
  print ("Files %s to %s" % (os.path.basename(filenames[0]),
      os.path.basename(filenames[-1])))
  exposures = [ Exposure(filename) for filename in filenames ]

  for e in exposures:
    e.filter_low_high(low_cutoff, high_cutoff)
    e.filter_neighbors(1)

  spectra = [ emission_spectrum(calib, e, low_energy, high_energy, energy_step, I0) for e, I0 in zip(exposures, I0s) ]

  return spectra


def interp_poisson(x, y, var, xp, yp, left=None, right=None):
  """
  Linearly interpolate points and calculate Poisson variance
  """
  i = -1
  n = len(xp)

  if y is None:
    y = zeros(len(x))

  if var is None:
    var = zeros(len(x))

  if left is None:
    left = yp[0]
  if right is None:
    right = yp[-1]

  for j in xrange(len(x)):
    if i >= n-1:
      y[j] = right
      continue

    skip = False
    while x[j] > xp[i+1]:
      i += 1
      if i >= n-1:
        y[j] = right
        skip = True
        break

    if skip:
      continue

    if i == -1:
      y[j] = left
    else:
      f = (x[j] - xp[i]) / (xp[i+1] - xp[i])
      y[j] = (1-f) * yp[i] + f * yp[i+1]
      var[j] = (1-f)*(1-f)*yp[i] + f*f*yp[i+1]

  return y,var


class EmissionSpectrum:
  def __init__(self):
    self.dataset_name = ""
    self.calibration_file = ""
    self.incident_energy = 0
    self.I0 = 1
    self.solid_angle_map_file = None
    self.solid_angle_map = None
    self.exposure_files = []
    self.filters = []

    spectrum = np.array([]).reshape(0,5)
    self.set_spectrum(spectrum)

    self.filename = None

    self.load_errors = []

  def set_incident_energy(self, incident_energy):
    self.incident_energy = incident_energy

  def set_I0(self, I0):
    self.I0 = I0

  def set_exposure_files(self, exposure_files):
    self.exposure_files = exposure_files

  def set_spectrum(self, spectrum):
    self.spectrum = spectrum

    self.emission = self.spectrum[:,0]
    self.intensity = self.spectrum[:,1]
    self.uncertainty = self.spectrum[:,2]
    self.raw_counts = self.spectrum[:,3]
    self.num_pixels = self.spectrum[:,4]
 
  def save(self, file=None, header_only=False):
    if file is None:
      file= self.filename

    file_opened = False

    if hasattr(file, 'write'):
      f = file
    else:
      f = open(file, 'w')
      self.filename = file
      file_opened = True

    try:
      f.write("# miniXS XES Spectrum\n#\n")
      f.write("# Dataset: %s\n" % self.dataset_name)
      f.write("# Calibration File: %s\n" % self.calibration_file)
      f.write("# Incident Energy: %.2f\n" % self.incident_energy)
      f.write("# I0: %.2f\n" % self.I0)
      if self.solid_angle_map_file:
        f.write("# Solid Angle Map: %s\n" % self.solid_angle_map_file)
      if self.filters:
        f.write("# Filters:\n")
        for fltr in self.filters:
          f.write('#   %s: %s\n' % (fltr.name, fltr.get_str()))
        f.write("#\n")

      f.write("# Exposures:\n")
      for ef in self.exposure_files:
        f.write("#   %s\n" % ef)
      f.write("#\n")

      if not header_only:
        if len(self.spectrum) > 0 and (len(self.spectrum.shape) != 2 and self.spectrum.shape[1] != 5):
          raise Exception("Invalid shape for spectrum array")

        if self.solid_angle_map is None:
          f.write("# E_emission    Intensity  Uncertainty  Raw_Counts   Num_Pixels\n")
          np.savetxt(f, self.spectrum, fmt=('%12.2f','%.6e','%.6e','% 11d',' % 11d'))
        else:
          f.write("# E_emission    Intensity  Uncertainty  Raw_Counts   Solid_Angle\n")
          np.savetxt(f, self.spectrum, fmt=('%12.2f','%.6e','%.6e','% 11d',' %.6e'))

    finally:
      # close the file if we opened it
      if file_opened:
        f.close()
      

  def load(self, filename=None, header_only=False):
    if filename is None:
      filename = self.filename
    else:
      self.filename = filename

    headers = []
    with open(filename, 'r') as f:
      pos = f.tell()
      line = f.readline()

      while line:
        if line[0] == "#":
          headers.append(line[2:])
        elif header_only:
          break
        else:
          f.seek(pos)
          spectrum = np.loadtxt(f)
          if len(self.spectrum.shape) == 1:
            spectrum.shape = (1,spectrum.shape[0])

          self.set_spectrum(spectrum)

        pos = f.tell()
        line = f.readline()

    parser = Parser({
      'Dataset': STRING,
      'Calibration File': STRING,
      'Incident Energy': FLOAT,
      'I0': FLOAT,
      'Solid Angle Map': STRING,
      'Filters': (LIST, STRING),
      'Exposures': (LIST, STRING)
      })
    parsed = parser.parse(headers)
    self.load_errors += parser.errors

    self.dataset_name = parsed.get('Dataset')
    self.calibration_file = parsed.get('Calibration File')
    self.incident_energy = parsed.get('Incident Energy', 0.0)
    self.I0 = parsed.get('I0', 0.0)
    solid_angle_map = parsed.get('Solid Angle Map')
    if solid_angle_map:
      self.load_solid_angle_map(solid_angle_map)
    self.exposure_files = parsed.get('Exposures')

    # load filters
    for filter_line in parsed.get('Filters', []):
      name,val = filter_line.split(':')
      name = name.strip()
      fltr = get_filter_by_name(name)
      if fltr == None:
        self.load_errors.append("Unknown Filter: '%s' (Ignoring)" % name)
      else:
        fltr.set_str(val.strip())
        self.filters.append(fltr)

    return len(self.load_errors) == 0

  def load_solid_angle_map(self, map_file):
    try:
      if os.path.exists(map_file):
        map = np.loadtxt(map_file)
      else:
        path = os.path.join(os.path.dirname(__file__), 'data', map_file)
        map = np.loadtxt(path)
    except IOError:
      raise IOError("Solid Angle Map File not found: '%s'. This must either be a full path, or relative to the minixs data directory." % map_file)

    self.solid_angle_map_file = map_file
    self.solid_angle_map = map

  def process(self, emission_energies=None, skip_columns=[]):
    calibration = minixs.calibrate.Calibration()
    calibration.load(self.calibration_file)

    exposure = Exposure()
    exposure.load_multi(self.exposure_files)

    for f in self.filters:
      f.filter(exposure.pixels, self.incident_energy)

    # generate emission energies from range of calibration matrix
    if emission_energies is None:
      Emin = calibration.calibration_matrix[np.where(calibration.calibration_matrix > 0)].min()
      Emax = calibration.calibration_matrix.max()

      emission_energies = np.arange(Emin,Emax,.1)

    spectrum = process_spectrum(calibration.calibration_matrix,
                                exposure,
                                emission_energies,
                                self.I0,
                                calibration.dispersive_direction,
                                calibration.xtals,
                                self.solid_angle_map,
                                skip_columns=skip_columns)

    self.set_spectrum(spectrum) 

  def process_binned(self, E1, E2, Estep):
    calib = minixs.calibrate.load(self.calibration_file)

    exposure = Exposure()
    exposure.load_multi(self.exposure_files)
    exposure.apply_filters(self.incident_energy, self.filters)
    spectrum = binned_emission_spectrum(calib.calibration_matrix,
                                        exposure,
                                        E1,
                                        E2,
                                        Estep,
                                        self.I0)
    self.set_spectrum(spectrum)




'''
 Copyright (c) 2019, UChicago Argonne, LLC
 See LICENSE file.
'''
import PyQt5.QtWidgets as qtWidgets
import logging
from PyQt5.QtCore import pyqtSignal, pyqtSlot
logger = logging.getLogger(__name__)


LABEL_NAME_STR = 'labelName'
ITEM_LIST_STR = 'itemList'

class FilterView(qtWidgets.QWidget):
    '''
    classdocs
    '''

    filterChanged = pyqtSignal(object, name="filterChanged")
    
    def __init__(self, *args, **kwargs):
        '''
        Constructor
        '''
        self.layout = qtWidgets.QHBoxLayout()
        self.labelName = None
        self.valueWidget = None
        if (LABEL_NAME_STR in kwargs):
            self.labelName = kwargs[LABEL_NAME_STR]
            del kwargs[LABEL_NAME_STR]
        if 'filter' in kwargs.keys():
            self.filter = kwargs['filter']
            del(kwargs['filter'])
        super(FilterView, self).__init__(*args, **kwargs)
        self.enableBox = qtWidgets.QCheckBox(self.filter.name)
        self.layout.addWidget(self.enableBox)
        self.setLayout(self.layout)
        self.enableBox.clicked.connect(self.filterEnableChanged)
        
    def setValue(self, value):
        pass 
    
    def getValue(self):
        return None

    def isEnabled(self):
        return self.enableBox.isEnabled()
    
    def text(self):
        return self.enableBox.text()
    
    def filterEnabled(self):
        return self.enableBox.isChecked()
    
    def filterEnableChanged(self, enabled):
        self.valueWidget.setEnabled(enabled)
        self.filterChanged.emit(self)
        
class IntFilterView(FilterView):
    
    def __init__(self, *args, **kwargs):
        super(IntFilterView, self).__init__(*args, **kwargs)
        self.valueWidget = qtWidgets.QSpinBox()
        self.layout.addWidget(self.valueWidget)
        self.valueWidget.setMaximum(10000000)
        self.valueWidget.setValue(self.filter.default_val)
        self.valueWidget.setEnabled(self.filter.default_enabled)
        self.enableBox.setChecked(self.filter.default_enabled)
        self.valueWidget.valueChanged.connect(self.handleValueChanged)
        
    def setValue(self, value):
        self.valueWidget.setValue(value)
    
    def getValue(self):
        return self.valueWidget.value()
    
    @pyqtSlot(int)
    def handleValueChanged(self, value):
        self.filterChanged.emit(self)
        
class StringFilterView(FilterView):
    
    def __init__(self, *args, **kwargs):
        super(StringFilterView, self).__init__(*args, **kwargs)
        self.valueWidget = qtWidgets.QLineEdit()
        self.layout.addWidget(self.valueWidget)
        self.valueWidget.setText(self.filter.default_val)
        self.valueWidget.setEnabled(self.filter.default_enabled)
        self.enableBox.setChecked(self.filter.default_enabled)
        self.valueWidget.editingFinished.connect(self.handleValueChanged)

    def setValue(self, value):
        self.valueWidget.setText(value)
        
    def getValue(self):
        return self.valueWidget.text()

    @pyqtSlot()
    def handleValueChanged(self):
        self.filterChanged.emit(self)
        
    
class ChoiceFilterView(FilterView):
    
    def __init__(self, * args, **kwargs):
#         if (ITEM_LIST_STR in kwargs):
#             self.labelName = kwargs[ITEM_LIST_STR]
#             del kwargs[ITEM_LIST_STR]
        super(ChoiceFilterView, self).__init__(*args, **kwargs)
        self.valueWidget = qtWidgets.QComboBox()
        logger.debug("Choices %s" % self.filter.CHOICES)
        #for choice in self.filter.CHOICES:
        self.valueWidget.addItems(self.filter.CHOICES)
        self.layout.addWidget(self.valueWidget)
#        self.valueWidget.setCurrentIndex(self.filter.default_val[0])
        self.valueWidget.setEnabled(self.filter.default_enabled)
        self.valueWidget.setCurrentIndex(0)
        self.enableBox.setChecked(self.filter.default_enabled)
        
        self.valueWidget.currentIndexChanged.connect(self.handleValueChanged)

    def setValue(self, value):
        self.valueWidget.setCurrentIndex(value)

    def getValue(self):
        return self.valueWidget.currentText()
        
    @pyqtSlot(int)
    def handleValueChanged(self, selectionIndex):
        self.filterChanged.emit(self)
        
        
REGISTRY = []

def register(fltr, view):
  global REGISTRY
  REGISTRY.append((fltr, view)) #, wx.NewId()

def filter_ids():
  global REGISTRY
  return [id for flter, view in REGISTRY] #, id



import wx


VIEW_MODE_COMBINED   = 0
VIEW_MODE_INDIVIDUAL = 1
VIEW_MODE_CALIB      = 2
VIEW_MODE_SPECTRUM   = 3

ID_MAIN_FRAME      = wx.NewId()

ID_DATASET         = wx.NewId()
ID_ENERGY          = wx.NewId()
ID_NORM            = wx.NewId()

ID_CALIB           = wx.NewId()
ID_CALIB_LOAD      = wx.NewId()
ID_SPECTROMETER    = wx.NewId()

ID_EXPOSURE_LIST   = wx.NewId()
ID_EXPOSURE_ADD    = wx.NewId()
ID_EXPOSURE_DEL    = wx.NewId()

ID_EXPOSURE_VIEW   = wx.NewId()

ID_FILTER_ADD      = wx.NewId()
ID_FILTER_REMOVE   = wx.NewId()

ID_VIEW_MODE       = wx.NewId()

ID_PROCESS         = wx.NewId()

ID_INDIVIDUAL_MIN  = wx.NewId()
ID_INDIVIDUAL_MAX  = wx.NewId()
ID_INDIVIDUAL_EXP  = wx.NewId()

ID_COMBINED_MIN    = wx.NewId()
ID_COMBINED_MAX    = wx.NewId()

STATUS_COORDS  = 0
STATUS_MESSAGE = 1



'''
 Copyright (c) 2019, UChicago Argonne, LLC
 See LICENSE file.
'''
import sys
import PyQt5.QtWidgets as qtWidgets
from minixs.gui.qtcalibrator.calibratormainwindow import CalibratorMainWindow
from minixs import filter
from minixs.gui.qthelpers import qtfilterview
        
def main():
  
    for f in filter.REGISTRY:
        if hasattr(f, 'view_name'):
            view_class = getattr(qtfilterview, f.view_name)
            qtfilterview.register(f, view_class)
            
    
    # Start the app.    
    app = qtWidgets.QApplication(sys.argv)
    mainWindow = CalibratorMainWindow()
    mainWindow.show()
    sys.exit(app.exec_())


import wx
from image_view import ImageView, EVT_COORDS
from image_tools import RangeTool, Crosshair, EVT_RANGE_CHANGED, EVT_RANGE_ACTION_CHANGED

if __name__ == "__main__":

  ID_HRANGE = wx.NewId()
  ID_VRANGE = wx.NewId()
  ID_BLINE = wx.NewId()
  ID_MULTIPLE  = wx.NewId()

  class DemoFrame(wx.Frame):
    def __init__(self, *args, **kwargs):
      wx.Frame.__init__(self, *args, **kwargs)

      box = wx.BoxSizer()
      p = DemoPanel(self, wx.ID_ANY)
      box.Add(p, 0, wx.EXPAND | wx.ALL, 5)

      self.SetSizerAndFit(box)

  ID_IMAGE_VIEW = wx.NewId()

  class DemoPanel(wx.Panel):
    def __init__(self, *args, **kwargs):
      wx.Panel.__init__(self, *args, **kwargs)

      vbox = wx.BoxSizer(wx.VERTICAL)

      im = ImageView(self, ID_IMAGE_VIEW, size=(400,300))
      vbox.Add(im, 1, wx.EXPAND | wx.BOTTOM)
      self.image = im

      self.box_tool = RangeTool(im)
      self.box_tool.direction = 0
      self.box_tool.active = True
      self.crosshair = Crosshair(im)

      c = wx.CheckBox(self, ID_HRANGE, 'Horizontal Range')
      vbox.Add(c, 0, wx.EXPAND | wx.BOTTOM, 5)

      c = wx.CheckBox(self, ID_VRANGE, 'Vertical Range')
      vbox.Add(c, 0, wx.EXPAND | wx.BOTTOM, 5)

      c = wx.CheckBox(self, ID_MULTIPLE, 'Multiple')
      vbox.Add(c, 0, wx.EXPAND)

      self.Bind(EVT_RANGE_ACTION_CHANGED, self.OnRangeActionChanged)
      self.Bind(EVT_RANGE_CHANGED, self.OnRangeChanged)
      self.Bind(EVT_COORDS, self.OnCoords)
      self.Bind(wx.EVT_CHECKBOX, self.OnCheck)

      self.SetSizerAndFit(vbox)

    def OnRangeActionChanged(self, evt):
      print ("action: %s" % evt.action)

    def OnRangeChanged(self, evt):
      print ("range: " % evt.range)

    def OnCoords(self, evt):
      if evt.x == None or evt.y == None:
        print ("coords: none")
      else:
        print ("coords: (%d,%d)" % (evt.x, evt.y))
    def OnCheck(self, evt):
      id = evt.GetId()
      checked = evt.IsChecked()

      if id == ID_VRANGE:
        self.crosshair.SetActive(dir != 0)
        self.crosshair.ToggleDirection(Crosshair.HORIZONTAL, checked)
        self.box_tool.ToggleDirection(RangeTool.VERTICAL, checked)

      elif id == ID_HRANGE:
        self.crosshair.SetActive(dir != 0)
        self.crosshair.ToggleDirection(Crosshair.VERTICAL, checked)
        self.box_tool.ToggleDirection(RangeTool.HORIZONTAL, checked)

      elif id == ID_MULTIPLE:
        self.box_tool.SetMultiple(checked)

  a = wx.App()
  f = DemoFrame(None)

  f.Show()
  a.MainLoop()



WILDCARD_CALIB = "Calibration Files (*.calib);;Data Files (*.dat);;Text Files (*.txt);;All Files(*)"
WILDCARD_EXPOSURE = "TIF Files (*.tif);;All Files(*)"
WILDCARD_XTAL = "All known types(*.xtal *.calib *.txt *.dat);;Crystal Files (*.xtal);;Calibration Files (*.calib);;Data Files (*.dat);;Text Files (*.txt);;All Files(*)"
WILDCARD_XTAL_EXPORT = "Crystal Files (*.xtal)"
WILDCARD_SCAN = "Scan Files (*.nnnn)|*.????|Text Files (*.txt);;All Files(*)"
WILDCARD_XES = "XES Data Files (*.xes);;Data Files (*.dat);;Text Files (*.txt)"



'''
Created on Mar 25, 2019

@author: hammonds
'''
import  wx
import  wx.lib.scrolledpanel as scrolled
from minixs.gui.image_view import ImageView

class TestPanel(scrolled.ScrolledPanel):
    def __init__(self, parent):
        scrolled.ScrolledPanel.__init__(self, parent, -1)
        vbox = wx.BoxSizer(wx.VERTICAL)
        desc = wx.StaticText(self, -1,
                            "ScrolledPanel extends wx.ScrolledWindow, adding all "
                            "the necessary bits to set up scroll handling for you.\n\n"
                            "Here are three fixed size examples of its use. The "
                            "demo panel for this sample is also using it -- the \nwxStaticLine "
                            "below is intentionally made too long so a scrollbar will be "
                            "activated."
                            )
        desc.SetForegroundColour("Blue")
        vbox.Add(desc, 0, wx.ALIGN_LEFT|wx.ALL, 5)
      
        self.SetSizer(vbox)
        self.SetAutoLayout(1)
        self.SetupScrolling()
app = wx.App(0)
frame = wx.Frame(None, wx.ID_ANY)
fa = ImageView(frame)
frame.Show()
app.MainLoop()


'''
 Copyright (c) 2019, UChicago Argonne, LLC
 See LICENSE file.
'''
from PyQt5.QtWidgets import QTableView, QAbstractItemView, \
    QStyledItemDelegate, QLineEdit
from PyQt5.QtCore import pyqtSignal

import logging
from PyQt5.QtGui import QDoubleValidator
from minixs.gui.qtcalibrator.model.exposurelistmodel import \
    ENERGY_COLUMN_NUMBER, EXPOSURE_COLUMN_NUMBER
from minixs import METHOD_ENTER_STR

logger=logging.getLogger(__name__)

ENERGY_ITEM_CHANGED_SIGNAL = "energyItemChanged"
EXPOSURE_FILE_NAME_CHANGED_SIGNAL = "exposureFilenameChanged"
SINGLE_EXPOSURE_NAME_CHANGED_SIGNAL = "singleExposureNameChanged"
SINGLE_ENERGY_CHANGED_SIGNAL = "singleEnergyChanged"


class ExposureItemDelegate(QStyledItemDelegate):
    '''
    Item Delegate used by the ExposureListView.  This provides for 
    speciql hansling of the information displayed.  
    Special cases handled:
        Column 0 is Energy, which is a real number.  IO for this is 
        QLineEdit with a QDoubleValidator applied.  This validator 
        allows only entry of Real numbers
        Column 1 is Filename of an exposure File.  IO for this is a 
        QLineEdit. Nothing special for this but maybe later could verify 
        valid files
    '''
    energyItemChanged = pyqtSignal(int, float, \
                                   name=ENERGY_ITEM_CHANGED_SIGNAL)
    exposureFileNameChanged = pyqtSignal(int, str, \
                                name=EXPOSURE_FILE_NAME_CHANGED_SIGNAL)
    def __init__(self, parent=None):
        super(ExposureItemDelegate, self).__init__(parent)
        self.parent = parent
        self.index = None
        
    def createEditor(self, parent, option, index):
        '''
        Implementation of the createEditor method for QItemDelegate
        '''
        logger.debug(METHOD_ENTER_STR)
        self.index = index
#         editor = QStyledItemDelegate.createEditor(self, parent, option, index)
        editor = QLineEdit(parent)
        if isinstance(editor, QLineEdit) and index.column() == \
                                                ENERGY_COLUMN_NUMBER:
            logger.debug("Using Energy instance")
            editor.setValidator(QDoubleValidator())
            editor.editingFinished.connect(self.handleFinishedEnergyInput)
        if isinstance(editor, QLineEdit) and index.column() == \
                                                EXPOSURE_COLUMN_NUMBER:
            logger.debug("Using Exposure File instance")
            editor.editingFinished.connect(self.handleFinishedExposureFileInput)
        return editor
    
    def handleFinishedEnergyInput(self):
        '''
        Handle input for Energy entries
        '''
        logger.debug(METHOD_ENTER_STR)
        editor = self.sender()
        logger.debug("senderIndex %s" % self.index.row())
        self.energyItemChanged.emit(self.index.row(), \
                                    float(editor.text()))
        
        
    def handleFinishedExposureFileInput(self):
        '''
        Handle input for ExposureFileEntries
        '''
        logger.debug(METHOD_ENTER_STR)
        editor = self.sender()
        self.exposureFileNameChanged.emit(self.index.row(), \
                                          editor.text())
        

class ExposureListView(QTableView):
    
    '''
    TableView for the list of Energies and exposures
    '''
    # Create signals created by this View
    singleExposureNameChanged = pyqtSignal(int, str, 
                               name=SINGLE_EXPOSURE_NAME_CHANGED_SIGNAL)
    singleEnergyChanged = pyqtSignal(int, float,
                               name=SINGLE_ENERGY_CHANGED_SIGNAL)

    def __init__(self, *args, **kwargs):
        '''
        Constructor
        '''
        super(ExposureListView, self).__init__(*args, **kwargs)
        self.horizontalHeader().setStretchLastSection(True)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.ContiguousSelection)
        self.setColumnWidth(1, 500)
        self.setMinimumWidth(600)
        self.setItemDelegate(ExposureItemDelegate())
        self.exposureItemDelegate = ExposureItemDelegate(self)
        self.exposureItemDelegate.energyItemChanged[int, float].connect(self.handleEnergyDelegateChanged)
        self.exposureItemDelegate.exposureFileNameChanged[int,str].connect(self.handleExposureFileNameDelegateChanged)
        self.setItemDelegate(self.exposureItemDelegate)
    
    def handleEnergyDelegateChanged(self, row, energy):
        '''
        Handle signals for the Energy Columns.  This allows manual entry 
        or edit of the info
        '''
        logger.debug(METHOD_ENTER_STR %  ((row, energy),))
        self.singleEnergyChanged.emit(row, energy)
        
    def handleExposureFileNameDelegateChanged(self, row, name):
        '''Handle Signals for the Exposure Files.  This allows manual 
        entry or edit of the info
        
        '''
        logger.debug(METHOD_ENTER_STR %((row, name),))
        self.singleExposureNameChanged.emit(row, name)
        
class ExposureControlView(QAbstractItemView):
    
    
    def __init__(self, *args, **kwargs):
        super(ExposureControlView, self).__init__(*args, **kwargs)
        



import os
from glob import glob
from minixs.parser import Parser, STRING, INT, FLOAT, LIST
import numpy as np
from numpy.linalg import norm
import minixs.geom
try:
    from itertools import izip as zip     # Python 2 way, For python 3 can use zip directly
except ImportError:
    pass                                  # Nothing to do for python 3
import logging
logger = logging.getLogger(__name__)

HBARC = 1973.2696 # eV * angstroms
HC    = 2 * np.pi * HBARC

DIR= os.path.join(os.path.dirname(__file__), 'data', 'spectrometers')

lattice_constants = {
    'Ge':  5.65735,
    'Si':  5.4309,
    'GaP': 5.4512,
    }

def clamp(v, min, max):
  if v < min:
    return min
  if v > max:
    return max
  else:
    return v

def tag_to_path(tag):
  return os.path.join(DIR, tag)

def list_spectrometers(include_names=False):
  files = glob(tag_to_path('*'))
  files.sort()
  if include_names:
    s = Spectrometer()
    tags = []
    names = []
    for f in files:
      try:
        s.load(f)
      except:
        print ("Invalid spectrometer file: %s" % f)
        continue
      tags.append(os.path.basename(f))

      name = s.name
      if not s.name:
        name = '%s %s' % (s.element, s.line)
      names.append(name)

    print ("tags: ", tags)
    print ("names: ", names)
    return tags, names

  else:
    return [ os.path.basename(f) for f in files]

class Spectrometer(object):
  def __init__(self, tag=None):
    if tag:
      self.load_by_tag(tag)
    logger.debug("initializing spectrometer camera_shape")
    self.camera_shape = (195, 487)
#     self.camera_shape = (1475, 1679)

  def load_by_tag(self, tag):
    self.load(tag_to_path(tag))

  def load(self, filename):
    self.load_errors = []

    with open(filename) as f:
      point_list = (LIST, (FLOAT, FLOAT, FLOAT))
      p = Parser({
        'Name': STRING,
        'Element': STRING,
        'Line': STRING,
        'Xtal': (STRING, INT, INT, INT),
        'Num Xtals': INT,
        'Energy Range': (FLOAT, FLOAT),
        'Exit Aperture': point_list,
        'Entrance Aperture': point_list,
        'Xtals': point_list,
        'Sample': point_list,
        'Camera': point_list,
        'Beam': point_list
        })

      info = p.parse(f.readlines())
      self.load_errors += p.errors

      self.name = info.get('Name')
      self.element = info.get('Element')
      self.line = info.get('Line')
      self.energy_range = info.get('Energy Range')

      xtal = info.get('Xtal')
      if xtal:
        self.xtal_type = info.get('Xtal')[0]
        self.xtal_cut = info.get('Xtal')[1:4]

        if self.xtal_type in lattice_constants:
          d0 = lattice_constants[self.xtal_type]
          self.E0 = HC * norm(self.xtal_cut) / (2 * d0)
        else:
          self.E0 = None

      else:
        self.load_errors.append('Missing crystal type information ("Xtal" line).')

      exit_aperture = info.get('Exit Aperture')
      if exit_aperture:
        if len(exit_aperture) == 4:
          self.exit_aperture = [geom.Point(*p) for p in exit_aperture]
        else:
          self.load_errors.append('Aperture contains %d points instead of 4.' % len(self.exit_aperture))
      else:
        self.load_errors.append("Missing Exit Aperture")

      entrance_aperture = info.get('Entrance Aperture')
      if entrance_aperture:
        if len(entrance_aperture) % 4 == 0:
          self.entrance_aperture = [
              [geom.Point(*p) for p in entrance_aperture[4*i:4*(i+1)]]
              for i in range(len(entrance_aperture)/4)
              ]
        else:
          self.load_errors.append('Aperture contains %d points instead of 4.' % len(self.entrance_aperture))
      else:
        self.load_errors.append('Missing Entrance Apertures')


      self.num_xtals = 0
      xtals = info.get('Xtals')
      if xtals:
        if len(xtals) % 4 == 0:
          self.xtals = [[geom.Point(*p) for p in xtals[4*i:4*(i+1)]] for i in range(len(xtals)/4)]
          self.xtal_planes = [geom.Plane.FromPoints(*x[0:3]) for x in self.xtals]
          self.num_xtals = len(self.xtals)
        else:
          self.load_errors.append('Xtal list contains %d points, which is not a multiple of 4.' % len(xtals))
      else:
        self.load_errors.append('Missing analyzer crystal geometry.')

      num_xtals = info.get('Num Xtals')
      if num_xtals:
        if self.num_xtals > 0 and num_xtals != self.num_xtals:
          self.load_errors.append('Geometry specified for %d xtals, but \'Num Xtals\' key also included with value: %d.' % (self.num_xtals, num_xtals))
        else:
          self.num_xtals = num_xtals

      sample = info.get('Sample')
      if sample:
        if len(sample) == 1:
          self.sample = geom.Point(*sample[0])
        else:
          self.load_errors.append('One sample point must be specified, not %d.' % len(sample))
      else:
        self.load_errors.append('Missing sample point.')

      camera = info.get('Camera')
      if camera:
        if len(camera) == 4:
          self.camera = [geom.Point(*p) for p in camera]
          self.camera_plane = geom.Plane.FromPoints(*self.camera[0:3])
        else:
          self.load_errors.append('Camera contains %d points instead of 4.' % len(self.camera))
      else:
        self.load_errors.append('Missing camera geometry')

      beam = info.get('Beam')
      if beam:
        if len(beam) == 1:
          self.beam = geom.Point(*beam[0])
        else:
          self.load_errors.append('One beam direction must be specified, not %d.' % len(beam))
      else:
        self.load_errors.append('Beam direction not specified')

  def camera_pixel_to_point(self, pixel):
    # returns the coordinates of the top left corner of the request pixel
    px,py = pixel
    h,w = self.camera_shape

    cam_x = self.camera[1] - self.camera[0]
    cam_y = self.camera[3] - self.camera[0]

    return self.camera[0] + px / float(w) * cam_x + py / float(h) * cam_y

  def point_to_camera_pixel(self, point, tolerance=.001):

    d = point - self.camera_plane.p0

    # check that point is within tolerance of plane
    if abs(np.dot(d,self.camera_plane.n)) > tolerance:
      return None # not in plane

    cx = self.camera[1] - self.camera[0]
    cy = self.camera[3] - self.camera[0]

    lx = norm(cx)
    cx /= lx

    ly = norm(cy)
    cy /= ly

    px = np.dot(d, cx) / lx * self.camera_shape[1]
    py = np.dot(d, cy) / ly * self.camera_shape[0]

    return (px,py)

  def camera_pixel_locations(self, dx=0.5, dy=0.5):
    """
    Calculate the coordinates of all pixels of the detector

    Parameters:
      dx - horizontal location within pixel (0 = left, .5 = center, 1=right)
      dy - vertical location within pixel (0 = top, .5 = center, 1=bottom)
    """
    h,w = self.camera_shape
    coords = np.zeros((w*h,2))
    index = np.arange(w*h)
    coords[:,0] = index % w
    coords[:,1] = index / w

    points = self.camera[0] + \
             (np.outer((coords[:,0]+dx)/float(w), self.camera[1] - self.camera[0]) +
              np.outer((coords[:,1]+dy)/float(h), self.camera[3] - self.camera[0]))
    return points.reshape((h,w,3))

  def image_points(self):
    """
    Calculate reflections of sample location about analyzer crystal faces
    """
    return [geom.reflect_through_plane(self.sample, xp) for xp in self.xtal_planes]

  def calculate_active_regions(self):
    """
    Find projection of sample through entrance apertures onto xtal planes.

    Currently requires number of entrance apertures to be same as number of xtals.
    """
    if len(self.entrance_aperture) != len(self.xtals):
      raise Exception("Number of entrance apertures and crystals must be same")

    # run through apertures and crystals
    for aperture, xtal_plane in zip(self.entrance_aperture, self.xtal_planes):
      for corner in aperture:
        l = geom.Line.FromPoints(self.sample, corner)
        p = geom.intersect_line_with_plane(l, xtal_plane)

  def project_point_through_rect_onto_camera(self, point, rect):
    """
    Find pixels covered by projection of a point through a rectangle
    """
    h, w = self.camera_shape
    x1 = w
    x2 = 0
    y1 = h
    y2 = 0

    for corner in rect:
      l = geom.Line.FromPoints(point, corner)
      p = geom.intersect_line_with_plane(l, self.camera_plane)

      x,y = self.point_to_camera_pixel(p)
      x = clamp(int(x+1e-5),0,w-1)
      y = clamp(int(y+1e-5),0,h-1)

      if x < x1:
        x1 = x
      if y < y1:
        y1 = y

      if x > x2:
        x2 = x
      if y > y2:
        y2 = y

    return [x1, y1, x2, y2]

  def calculate_projection_bounds(self):
    """
    Determine exposed region on camera from each xtal.

    Finds projection of specular radiation from source point
    through entrance aperture, off xtal, through exit aperture
    and onto camera.

    For now, this finds top left and bottom right corners (in camera coords)
    and assumes region to be rectangular.

    TODO: should be easy to extend this to calculate full trapezoidal region
          or, could add option to give largest rect inside, or smallest rect
          outside...
    """

    bounds = []

    images = self.image_points()

    for xtal_plane, image, entrance_aperture in zip(self.xtal_planes, images, self.entrance_aperture):
      # project image points through exit aperture onto camera
      exit_projection = self.project_point_through_rect_onto_camera(image, self.exit_aperture)

      # find region of crystals exposed by source
      active_region = [
          geom.intersect_line_with_plane(
            geom.Line.FromPoints(self.sample, corner),
            xtal_plane
            )
          for corner in entrance_aperture
          ]

      # project images through active region on to camera
      active_projection = self.project_point_through_rect_onto_camera(image, active_region)

      # "intersect" (not complete intersection... see TODO above)
      x1 = max(exit_projection[0], active_projection[0])
      y1 = max(exit_projection[1], active_projection[1])
      x2 = min(exit_projection[2], active_projection[2])
      y2 = min(exit_projection[3], active_projection[3])

      # add it to the list
      bounds.append([x1,y1,x2,y2])

    return bounds

  def mockup_calibration_matrix(self, dx=0.5, dy=0.5):
    """
    Create a theoretical calibration matrix for the designed spectrometer
    geometry.
    """
    h,w = self.camera_shape
    calib = np.zeros((h,w))

    d0 = lattice_constants[self.xtal_type]
    # XXX this assumes the crystal is cubic (all that we currently use are)
    #     it would be good to generalize this though
    d = d0 / norm(self.xtal_cut)

    images = self.image_points()
    bounds = []

    pixels = self.camera_pixel_locations(dx,dy)

    # find image points and xtal projection boundaries
    for xtal_plane, image, entrance_aperture in zip(self.xtal_planes, images, self.entrance_aperture):
      exit_projection = self.project_point_through_rect_onto_camera(image, self.exit_aperture)

      active_region = [
          geom.intersect_line_with_plane(
            geom.Line.FromPoints(self.sample, corner),
            xtal_plane
            )
          for corner in entrance_aperture
          ]

      active_projection = self.project_point_through_rect_onto_camera(image, active_region)

      x1 = max(exit_projection[0], active_projection[0])
      y1 = max(exit_projection[1], active_projection[1])
      x2 = min(exit_projection[2], active_projection[2])
      y2 = min(exit_projection[3], active_projection[3])

      bounds.append([x1,y1,x2,y2])
      rw = x2-x1
      rh = y2-y1

      dn = pixels[y1:y2, x1:x2].reshape((rw*rh,3)) - image
      length = np.sqrt((dn**2).sum(1))
      cos_theta = np.abs((dn*xtal_plane.n).sum(1)) / length
      energy = HC / (2 * d) / cos_theta
      energy = energy.reshape((rh,rw))

      calib[y1:y2,x1:x2] = energy

    self.images = images
    self.projection_bounds = bounds
    return calib

  def solid_angle_map(self, bounds=None):
    """
    Calculate the solid angle subtended by each pixel in sr

    Parameters:
      bounds - list of (x1,y1,x2,y2) bounding rectangles

    Each rectangle in the list of bounds should correspond to the
    projection from a single analyzer crystal through the exit aperture.

    The center of the bounds rect is used to determine which xtal it
    corresponds to. So, this could be incorrect if a small sliver at the
    edge of a crystal projection is provided.
    """

    xtals_reversed = False

    h,w = self.camera_shape
    domega = np.zeros((h,w))

    # calculate pixel size
    pw = norm(self.camera[1] - self.camera[0]) / w
    ph = norm(self.camera[3] - self.camera[0]) / h
    print (pw, ph)

    images = self.image_points()

    design_bounds = self.calculate_projection_bounds()
    if bounds is None:
      bounds = design_bounds

    # determine if xtals in design file are left to right on camera
    # or right to left
    # TODO: this will need to be updated to support spectrometers with
    #       multiple rows of crystals...
    if design_bounds[0][0] > design_bounds[-1][0]:
      xtals_reversed = True

    pixels = self.camera_pixel_locations()
    num_xtals = len(self.xtal_planes)

    for x1,y1,x2,y2 in bounds:
      xc = (x1+x2)/2.0
      yc = (y1+y2)/2.0

      # determine which crystal this corresponds to
      i = int(xc) * num_xtals / w
      if xtals_reversed:
        i = num_xtals - i - 1

      #print (xc,yc, i, self.xtals[i][0])
      # projection shape size
      sh = y2 - y1
      sw = x2 - x1

      dn = pixels[y1:y2,x1:x2].reshape((sw*sh,3)) - images[i]
      length = np.sqrt((dn**2).sum(1))
      cos_theta = np.abs((dn*self.camera_plane.n).sum(1)) / length
      domega[y1:y2,x1:x2] = (pw*ph*cos_theta / length**2).reshape((sh,sw))

    return domega



"""
Raw detector exposures
"""
from PIL import Image
import numpy as np
import os
import logging
try:
    from itertools import izip as zip     # Python 2 way, For python 3 can use zip directly
except ImportError:
    pass                                  # Nothing to do for python 3


logger = logging.getLogger(__name__)
class Exposure:
  """
  Raw detector exposures

  This handles loading and saving exposure files, which are currently assumed
  to be tif files from a Dectris Pilatus detector. Any image file that
  is supported by the Python Imaging Library should load correctly, however,
  unless the image has a single channel, it will most likely not work with
  any of the rest of the minixs code.
  """
  def __init__(self, filename = None):
    self.loaded = False
    if filename is not None:
      self.load(filename)

  def load(self, filename):
    """
    Load a single image file
    """
    self.filename = filename

    ext = filename.split(os.path.extsep)[-1]
    if ext == 'raw':
      with open(filename) as f:
        d = f.read()

      # XXX this assumes pilatus 100K...
      self.image = None
      self.raw = np.fromstring(d, '>f').astype('int32').reshape((195,-1))
      pass

    else:
      logger.debug("Opening image file %s" % filename)
      self.image = Image.open(filename)
      logger.debug("Image size: %s" % (self.image.size,))
      logger.debug("Image Min/Max %f, %f" (np.min(self.image), np.max(self.image)))
      self.raw = np.asarray(self.image)

    self.pixels = self.raw.copy()
    try:
      self.info = self.parse_description(self.image.tag.get(270))
    except:
      pass
    self.loaded = True

  def load_multi(self, filenames):
    """
    Load several image files summing together their pixel values
    """
    self.filenames = filenames
    self.pixels = None
    if filenames:
      self.loaded = True

    for f in filenames:
      ext = f.split(os.path.extsep)[-1]
      if ext == 'raw':
        with open(f) as fh:
          d = fh.read()

        # XXX this assumes pilatus 100K...
        p = np.fromstring(d, '>f').astype('int32').reshape((195,-1))
      else:
        logger.debug("Opening image file %s" % f)
        im = Image.open(f)
        logger.debug("Image size: %s" % (im.size,))
        p = np.asarray(im)

      if self.pixels is None:
        self.pixels = p.copy()
      else:
        self.pixels += p

  def parse_description(self, desc):
    try:
      # split into lines and strip off '#'
      info = [line[2:] for line in desc.split('\r\n')][:-1]
    except:
      info = []
    return info

  def apply_filters(self, energy, filters):
    """
    Apply a set of filters to an exposure

    Each filter in the list `filters` has its `filter` method called
    on this exposure's pixel array.
    """
    for f in filters:
      f.filter(self.pixels, energy)
  
  def filter_low_high(self, low, high):
    """
    DEPRECATED
    """
    if low == None: low = self.pixels.min()
    if high == None: high = self.pixels.max()

    mask = logical_and(self.pixels >= low, self.pixels <= high)
    self.pixels *= mask

  def filter_neighbors(self, cutoff):
    """
    DEPRECATED
    """
    from itertools import product
    nbors = (-1,0,1)
    mask = sum([
      roll(roll(self.pixels>0,i,0),j,1)
      for i,j in product(nbors,nbors)
      if i != 0 or j != 0
      ], 0) >= cutoff
    self.pixels *= mask

  def filter_bad_pixels(self, bad_pixels):
    """
    DEPRECATED
    """
    for x,y in bad_pixels:
      self.pixels[y,x] = 0


  def detect_bad_pixels(self, num_bins=20, bin_sep=3):
    raise Exception("Not yet implemented")
    # if we find bins separated from the rest by more than bin_sep zero bins, assume they contain bad pixels

    h = np.histogram(self.pixels.flat, num_bins)
    counts = h[0]
    bounds = h[1]

    print (counts)
    # start at high count end of histogram.
    high_start = -1
    zero_start = -1
    for i in range(len(counts))[::-1]:
      if counts[i] == 0 and high_start == -1:
        high_start = i+1
      elif counts[i] != 0 and high_start != -1:
        zero_start = i+1
        break

    num_zeros = high_start - zero_start
    print (high_start, zero_start, num_zeros)
    if num_zeros > bin_sep:
      max_good_counts = bounds[high_start]
      y,x = np.where(self.pixels > max_good_counts)
      bad_pixels = [(xi,yi) for xi,yi in zip(x,y)]
    else:
      bad_pixels = []

    return bad_pixels

  def detect_bad_pixels2(self):
    p = self.pixels.copy()
    n = np.product(p.shape)
    bad_pixels = []
    while True:
      a,b = np.histogram(p.flat)

      if sum(a==0) < 2:
        break

      i = np.where(p > b[-2])
      bad_pixels += [(x,y) for y,x in zip(*i)]
      p[i] = 0

    print (a)
    print (b)
    return bad_pixels, p




import wx
import os
from const import *
import minixs as mx
from minixs.gui.wildcards import *
from minixs.gui.file_dialog import FileDialog
from minixs.gui.image_view import EVT_COORDS
from minixs.gui.image_tools import Crosshair
from matplotlib.cm import jet, gray
from matplotlib.colors import Normalize

BG_VALID = '#ffffff'
BG_INVALID = '#ffdddd'

ERROR_ENERGY    = 0x1
ERROR_NORM      = 0x2
ERROR_CALIB     = 0x4
ERROR_EXPOSURES = 0x8
ERROR_FILTERS   = 0x10

class ProcessorController(object):
  def __init__(self, view, model):
    """
    Initialize
    """
    self.view = view
    self.model = model

    self.dialog_dirs = {
        'last': '',
        }

    self.calib = ''
    self.combined_exposure = mx.exposure.Exposure()
    self.exposures = []

    self.spectrometer_tags, self.spectrometer_names = mx.spectrometer.list_spectrometers(include_names = True)
    self.spectrometer_names.append("Other")
    self.view.SetSpectrometerNames(self.spectrometer_names)
    self.view.SetSpectrometer(len(self.spectrometer_names) - 1)

    self.combined_range = [0,100]
    self.individual_range = [0,100]
    self.exposure_num = 0

    self.changed = False
    self.spectrum_invalid = True
    self.error = 0

    # setup image view tools
    self.view.image_view.zoom_on_wheel = True
    self.crosshair = Crosshair(self.view.image_view)
    self.crosshair.SetActive(True)
    self.crosshair.ToggleDirection(Crosshair.HORIZONTAL | Crosshair.VERTICAL, True)


    a = wx.AboutDialogInfo()
    a.SetName("miniXS XES Processor")
    a.SetDescription("Mini X-ray Spectrometer Emission Spectrum Processor")
    a.SetVersion("0.0.1")
    a.SetCopyright("(c) Seidler Group 2011")
    a.AddDeveloper("Brian Mattern (bmattern@uw.edu)")
    self.about_info = a

    self.BindCallbacks()

    wx.FutureCall(0, self.SetViewMode)

  def BindCallbacks(self):
    """
    Connect up event handlers
    """
    callbacks = [
        (wx.EVT_CLOSE, [ (ID_MAIN_FRAME, self.OnClose) ]),
        (wx.EVT_MENU, [
          (wx.ID_EXIT, self.OnExit),
          (wx.ID_OPEN, self.OnOpen),
          (wx.ID_SAVE, self.OnSave),
          (wx.ID_ABOUT, self.OnAbout),
          ]),
        (wx.EVT_TEXT, [
          (ID_DATASET, self.OnDataset),
          (ID_ENERGY, self.OnEnergy),
          (ID_NORM, self.OnNorm),
          ]),
        (wx.EVT_BUTTON, [
          (ID_CALIB_LOAD, self.OnLoadCalibration),
          (ID_EXPOSURE_ADD, self.OnAddExposures),
          (ID_EXPOSURE_DEL, self.OnDeleteSelected),
          (ID_PROCESS, self.OnProcess),
          ]),
        (wx.EVT_RADIOBOX, [
          (ID_VIEW_MODE, self.OnViewMode),
          ]),
        (wx.EVT_SPINCTRL, [
          (ID_INDIVIDUAL_MIN, self.OnIndividualMin),
          (ID_INDIVIDUAL_MAX, self.OnIndividualMax),
          (ID_INDIVIDUAL_EXP, self.OnIndividualExp),
          (ID_COMBINED_MIN, self.OnCombinedMin),
          (ID_COMBINED_MAX, self.OnCombinedMax),
          ]),
        (EVT_COORDS, [ (ID_EXPOSURE_VIEW, self.OnImageCoords), ]),
        ]

    for event, bindings in callbacks:
      for id, callback in bindings:
        self.view.Bind(event, callback, id=id)


  def OnClose(self, evt):
    """
    Window close handler

    Checks if model has changed since last save and notifies user
    if it has.
    """
    if not evt.CanVeto():
      self.view.Destroy()
      return True

    self.view.Destroy()

  def OnExit(self, evt):
    """
    File > Exit handler
    """
    self.view.Close()

  def OnSave(self, evt):
    header_only = False
    if self.spectrum_invalid == True:
      errdlg = wx.MessageDialog(self.view, "Warning: You have changed parameters since last processing the spectrum. Saving now will only save the parameters, and not the spectrum itself.", "Error", wx.OK | wx.CANCEL | wx.ICON_WARNING)
      ret = errdlg.ShowModal()
      errdlg.Destroy()

      if ret == wx.ID_OK:
        header_only = True
      else:
        return

    filename = FileDialog(
        self.view,
        self.dialog_dirs,
        'xes',
        'Enter a filename to save to',
        wildcard=WILDCARD_XES,
        save=True
        )

    if not filename:
      return

    # check if file exists
    if os.path.exists(filename):
      errdlg = wx.MessageDialog(self.view, "This will overwrite the file:\n%s" % filename, "Warning", wx.OK | wx.CANCEL | wx.ICON_WARNING)
      ret = errdlg.ShowModal()
      errdlg.Destroy()

      if ret != wx.ID_OK:
        return

    # save the file
    self.model.save(filename, header_only=header_only)

  def OnOpen(self, evt):
    filename = FileDialog(
        self.view,
        self.dialog_dirs,
        'xes',
        'Select an xes file to open',
        wildcard=WILDCARD_XES
        )

    if not filename:
      return

    self.load(filename)
    self.SetViewMode(VIEW_MODE_SPECTRUM)

  def ModelToView(self):

    self.view.dataset_entry.SetValue(self.model.dataset_name)
    self.view.energy_entry.SetValue(str(self.model.incident_energy))
    self.view.norm_entry.SetValue(str(self.model.I0))

    self.exposures = []
    for f in self.model.exposure_files:
      base = os.path.basename(f)
      self.view.exposure_listbox.AppendAndEnsureVisible(base)
      self.exposures.append(mx.exposure.Exposure(f))
    self.combined_exposure.load_multi(self.model.exposure_files)
    self.view.SetExposureCount(len(self.exposures))
    self.SetCalibrationFilename(self.model.calibration_file)
    self.view.axes.cla()
    self.view.axes.plot(self.model.emission, self.model.intensity)

  def OnAbout(self, evt):
    """
    Help > About handler
    """
    wx.AboutBox(self.about_info)

  def OnDataset(self, evt):
    self.model.dataset_name = evt.GetString()

  def OnEnergy(self, evt):
    s = evt.GetString()
    valid = True
    if s:
      try:
        self.model.incident_energy = float(s)
        self.error &= ~ERROR_ENERGY
      except ValueError:
        self.error |= ERROR_ENERGY
        valid = False
    else:
      self.model.incident_energy = 0

    evt.GetEventObject().SetOwnBackgroundColour(BG_VALID if valid else BG_INVALID)

  def OnNorm(self, evt):
    s = evt.GetString()
    valid = True
    if s:
      try:
        self.model.I0 = float(s)
        self.error &= ~ERROR_NORM
      except ValueError:
        self.error |= ERROR_NORM
        valid = False
    else:
      self.model.I0 = 1

    evt.GetEventObject().SetOwnBackgroundColour(BG_VALID if valid else BG_INVALID)
    self.InvalidateSpectrum()

  def OnLoadCalibration(self, evt):
    filename = FileDialog(
        self.view,
        self.dialog_dirs,
        'calib',
        'Select a calibration file to open',
        wildcard=WILDCARD_CALIB
        )

    if not filename:
      return

    self.SetCalibrationFilename(filename)
    if len(self.exposures) == 0:
      self.SetViewMode(VIEW_MODE_CALIB)

  def SetCalibrationFilename(self, filename):
    try:
      calib = mx.calibrate.load(filename)
    except mx.filetype.InvalidFileError:
      errmsg = "The selected file was not a valid calibration file." 
      errdlg = wx.MessageDialog(self.view,  errmsg, "Error", wx.OK | wx.ICON_WARNING)
      errdlg.ShowModal()
      errdlg.Destroy()
      return

    self.calib = calib
    self.model.calibration_file = filename

    self.view.calibration_file_entry.SetValue(os.path.basename(filename))

    if calib.load_errors:
      errmsg = "Warning: the following errors were encountered while loading the calibration file:\n\n  %s.\n\nThese may or may not prevent this calibration matrix from being used." % '\n  '.join(calib.load_errors)
      errdlg = wx.MessageDialog(self.view,  errmsg, "Error", wx.OK | wx.ICON_WARNING)
      errdlg.ShowModal()
      errdlg.Destroy()

  def SetViewMode(self, mode=VIEW_MODE_COMBINED):
    self.view_mode = mode

    if mode == VIEW_MODE_COMBINED:
      print ('combined')
      if self.combined_exposure.loaded:
        p = Normalize(*self.combined_range)(self.combined_exposure.pixels)
        self.view.image_view.SetPixels(p, gray)
      else:
        print ('not yet loaded')

    elif mode == VIEW_MODE_INDIVIDUAL:
      print ('individual')
      if self.exposures:
        p = Normalize(*self.individual_range)(self.exposures[self.exposure_num].pixels)
        self.view.image_view.SetPixels(p, gray)
      else:
        print ('no exposures')
    elif mode == VIEW_MODE_CALIB:
      print ('calib')
      if self.calib:
        p = Normalize(*self.calib.energy_range())(self.calib.calibration_matrix)
        self.view.image_view.SetPixels(p, jet)
      else:
        pass
    elif mode == VIEW_MODE_SPECTRUM:
      print ('spectrum')
      pass

    self.view.SetViewMode(mode)

  def UpdateCoordStatus(self, cx, cy):
    if cx is None or cy is None:
      self.view.SetStatusText('', STATUS_COORDS)
      return

    cx = int(cx)
    cy = int(cy)
    try:
      if self.view_mode == VIEW_MODE_COMBINED:
        if not self.combined_exposure.loaded:
          return
        z = "%d" % self.combined_exposure.pixels[cy,cx]

      elif self.view_mode == VIEW_MODE_INDIVIDUAL:
        if not self.exposures:
          return
        z = "%d" % self.exposures[self.exposure_num].pixels[cy,cx]
      elif self.view_mode == VIEW_MODE_CALIB:
        if not self.calib:
          return
        z = "%.2f" % self.calib.calibration_matrix[cy,cx]
      else:
        return
    except IndexError:
      z = "N/A"

    s = ("(%d, %d) -> "% (cx,cy)) + z
    self.view.SetStatusText(s, STATUS_COORDS)

  def OnAddExposures(self, evt):
    filenames = FileDialog(
        self.view,
        self.dialog_dirs,
        'exposure',
        'Select exposure file(s)',
        wildcard=WILDCARD_EXPOSURE,
        multiple=True
        )

    if not filenames:
      return

    invalid_files = []

    for f in filenames:
      try:
        e = mx.exposure.Exposure(f)
      except IOError:
        invalid_files.append(f)
        continue

      self.model.exposure_files.append(f)
      base = os.path.basename(f)
      self.view.exposure_listbox.AppendAndEnsureVisible(base)
      self.exposures.append(e)

    self.combined_exposure.load_multi(self.model.exposure_files)
    self.view.SetExposureCount(len(self.exposures))

    if invalid_files:
      errmsg = "The following files were not recognized:\n\n  " + '\n  '.join(invalid_files)
      errdlg = wx.MessageDialog(self.view,  errmsg, "Error", wx.OK | wx.ICON_WARNING)
      errdlg.ShowModal()
      errdlg.Destroy()

  def OnDeleteSelected(self, evt):
    sel = list(self.view.exposure_listbox.GetSelections())
    # run through indices backwards and remove from lists
    sel.sort(reverse=True)
    for i in sel:
      del(self.model.exposure_files[i])
      del(self.exposures[i])
      self.view.exposure_listbox.Delete(i)

    self.combined_exposure.load_multi(self.model.exposure_files)

  def OnProcess(self, evt):
    if self.model.validate():
      self.model.process()
      self.spectrum_invalid = False
      self.view.axes.cla()
      self.view.axes.plot(self.model.emission, self.model.intensity)
      self.SetViewMode(VIEW_MODE_SPECTRUM)
    else:
      errors = '\n'.join('%4d. %s' % (i+1,err) for i,err in enumerate(self.model.validation_errors))
      errmsg = "The following errors prevented this spectrum from being processed:\n\n%s\n" % errors
      dlg = wx.MessageDialog(self.view, errmsg, 'Error', wx.OK | wx.ICON_ERROR)
      dlg.ShowModal()

  def OnViewMode(self, evt):
    mode = self.view.view_mode.GetSelection()
    self.SetViewMode(mode)

  def OnIndividualMin(self, evt):
    self.individual_range[0] = evt.GetInt()
    self.UpdateImageView()

  def OnIndividualMax(self, evt):
    self.individual_range[1] = evt.GetInt()
    self.UpdateImageView()

  def OnIndividualExp(self, evt):
    self.exposure_num = evt.GetInt() - 1
    self.SetViewMode(self.view_mode)

  def OnCombinedMin(self, evt):
    self.combined_range[0] = evt.GetInt()
    self.UpdateImageView()

  def OnCombinedMax(self, evt):
    self.combined_range[1] = evt.GetInt()
    self.UpdateImageView()

  def OnImageCoords(self, evt):
    x, y = evt.x, evt.y
    self.UpdateCoordStatus(x,y)

  def UpdateImageView(self):
    self.SetViewMode(self.view_mode)

  def InvalidateSpectrum(self):
    """
    Spectrum needs to be reprocessed
    """
    self.spectrum_invalid = True

  def Changed(self):
    """
    Something has changed since last save
    """
    self.changed = True

  def load(self, filename):
    # XXX handle errors...
    self.model.load(filename)
    self.ModelToView()



'''
Created on Mar 25, 2019

@author: hammonds
'''
import unittest
from minixs.gui import util

class Test(unittest.TestCase):


    def setUp(self):
        self.old_column_count = 8
        self.filename_olddet = "../data/Cuecal.0001"
        self.new_column_count = 23
        self.filename_newdet = "../data/mini2CrCali.0001"



    def testOldDetFile(self):
        column_names = util.read_scan_column_names(self.filename_olddet)
        self.assertEqual(len(column_names), self.old_column_count)
        print(column_names)
        
    def testNewDetFile(self):
        column_names = util.read_scan_column_names(self.filename_newdet)
        self.assertEqual(len(column_names), self.new_column_count)
        print(column_names)
    

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


"""
Miniature X-ray Spectrometer (miniXS) Tools
"""
import os
import logging
import logging.config
try:
    from configparser import NoSectionError      # Python 3 way
except:
    from ConfigParser import NoSectionError      # Python 2 way

__version__ = '0.8'
LOGGER_NAME="minixs"
METHOD_ENTER_STR = "Enter %s\n-------------------"
METHOD_EXIT_STR = "Exit %s\n---------------------"
LOGGER_DEFAULT = {
    'version' : 1,
    'handlers' : {'consoleHandler' : {'class' : 'logging.StreamHandler',
                               'level' : 'INFO',
                               'formatter' : 'consoleFormat',
                               'stream' : 'ext://sys.stdout'} ,
                  },
    'formatters' : {'consoleFormat' : {'format' : '%(asctime)-15s - %(name)s - %(funcName)s- %(levelname)s - %(message)s'},
                    },
    'loggers' : {'root' :{'level' : 'INFO',
                        'handlers' : ['consoleHandler',],
                      },
               LOGGER_NAME : {'level' : 'INFO',
                            'handlers' : ['consoleHandler',],
                            'qualname' : LOGGER_NAME
                            }
               },
   }

userDir = os.path.expanduser("~")
logConfigFile = os.path.join(userDir, LOGGER_NAME + 'Log.config')
if os.path.exists(logConfigFile):
    print ("logConfigFile " + logConfigFile )
    try:
        logging.config.fileConfig(logConfigFile, disable_existing_loggers=False)
        print("Success Openning logfile")
    except (NoSectionError,TypeError) as ex:
        print ("In Exception to load dictConfig package %s\n %s" % (LOGGER_NAME, ex))
        logging.config.dictConfig(LOGGER_DEFAULT)
    except KeyError as ex:
        print ("logfile %s was missing or had errant sections %s" %(logConfigFile, ex.args))

else:
    logging.config.dictConfig(LOGGER_DEFAULT)

import minixs.calibrate as calibrate, \
       minixs.emission as emission, \
       minixs.exposure, \
       minixs.filter, \
       minixs.misc, \
       minixs.rixs, \
       minixs.scanfile, \
       minixs.spectrometer

from minixs.constants import *

__all__ = [
  'calibrate',
  'emission',
  'exposure',
  'filter',
  'misc',
  'rixs',
  'scanfile',
  'spectrometer',
  ]


